package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import mro.util.*;
import mro.util.properties.*;

import java.util.Vector; 

/**
 * GroupingAlgorithm. 
 *      
 * @version          060309
 * @author           Magnus Rosell
 */
public abstract class GroupingAlgorithm 
    implements HasProperties, UniqueIdentifiable{

    // Dpa om till IGroupingAlgorithm???

    protected Properties properties;
    protected UniqueIdentifier identifier;

    protected boolean printGroupSizesInIterations = true;
    

    public GroupingAlgorithm(){
	this.properties = getDefaultProperties();	
    }

    /*
    public void setPrintGroupSizesInIterations(boolean print){
	printGroupSizesInIterations = print;
    }
    */

    /*
    // Implementera basala Propery:s hr.
    // "rv" i underklasser.
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Grouping Algorithm Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Number of clusters", 5, "Number of clusers");
	    properties.addProperty(p);

	    p = new Property("SimSort", true, "Sort objects in clusters according to similarity to cluster centroid.");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }
    */


    // IObjectSet, varfr inte IObjectGroup?
    /**
     * Clusters the entire IObjectSet. Stores the clustering 
     * in the IObjectSet.
     */
    public abstract IObjectGrouping cluster(IObjectSet set);

    /**
     * Clusters the IObjectGroup. Does not store the result.
     */
    public abstract IObjectGrouping cluster(IObjectGroup group);

    public abstract String getInfoString();

    public abstract String getAlgorithmName();
    
    public abstract Vector getPropertyVector();




    /**
     * For debuggin purposes!
     */	
    protected void printGroupSizes(Vector<IObjectGroup> groups){
	for(int i=0; i<groups.size(); i++){
	    System.out.print(groups.elementAt(i).numberOfIObjects()+" ");
	}
	System.out.println();
    }
	
	
    /**
     * For debuggin purposes!
     */	
    protected void printGroup(IObjectGroup group){
	System.out.println(group.getString());
	Vector ordered = group.getOrdered();
	for(int i=0; i<ordered.size(); i++)
	    System.out.println(" * "+((IObject) ordered.elementAt(i)).getString());
    }

}
