package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;
/*
  import java.util.Vector; 
  import java.util.Map.Entry; 
  import java.util.HashMap; 
  import java.util.HashSet; 
  import java.util.Set; 
*/

/**
 * IVectorCategorizer
 *      
 * @version          080702
 * @author           Magnus Rosell
 */
public class IVectorCategorizer extends GroupingAlgorithm implements NeedsSparseISimilarity{

    public static int ivectorCategorizerGroupingNumber = 1;
	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("IVectorCategorizer Properties", CurrentClassGetter.getClassName());
		
	    /*			
	try{

	    Property p = new Property("Number of clusters", 5, "Number of clusters");
	    properties.addProperty(p);

	    p = new Property("Init Type", 1, "Kind of initialization");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "\"Number of clusters\" random texts as start clusters");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Random clustering as start clusters.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);
			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	    */

	return properties;
    }

    private SparseISimilarity similarity;
    private Vector<IVector<IVectorIndex>> vectors;

    public IVectorCategorizer(){
	this.properties = getDefaultProperties();	
    }

    public IVectorCategorizer(mro.util.properties.Properties properties){
	this.properties=properties;
    }

    public IVectorCategorizer(SparseISimilarity similarity, 
			      Vector<IVector<IVectorIndex>> vectors){
	this.similarity = similarity;
	this.vectors = vectors;
	this.properties = getDefaultProperties();	
    }

    public IVectorCategorizer(SparseISimilarity similarity, 
			      Vector<IVector<IVectorIndex>> vectors,
			      mro.util.properties.Properties properties){
	this.similarity = similarity;
	this.vectors = vectors;
	this.properties=properties;
    }
	
    public void setSimilarity(SparseISimilarity similarity){
	this.similarity = similarity;
    }


    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}

	return grouping;
    }

    public IObjectGrouping cluster(IObjectGroup iogroup){
	
	Vector<IObjectGroup> groups = new Vector<IObjectGroup>();
	for(int i=0; i<vectors.size(); i++){
	    IObjectGroup group = new IObjectGroup("Group "+(i+1));
	    groups.add(group);
	}
		
	Vector<IObject> ordered = iogroup.getOrdered();
	for(int o=0; o<ordered.size(); o++){
	    IObject io = ordered.get(o);
	    IVector<IVectorIndex> iorv = similarity.getRepresentation(io);
	    IVector<IVectorIndex> vec = vectors.get(0);
	    float bestSim = similarity.sim(iorv, vec);
	    int bestGroupNr = 0;
	    for(int i=1; i<vectors.size(); i++){
		vec = vectors.get(i);
		float sim = similarity.sim(iorv, vec);
		if(sim>bestSim){
		    bestSim=sim;
		    bestGroupNr = i;
		}
	    }
	    IObjectGroup bestGroup = groups.get(bestGroupNr);
	    bestGroup.add(io);
	}
		
	String name = "IVectorCategorizer Grouping "+ivectorCategorizerGroupingNumber;
	ivectorCategorizerGroupingNumber++;
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    grouping.add(group);
	    //	    printGroup(group);
	}
		
	return grouping;
    }





	
	
    public String getInfoString(){
    	return "IVectorCategorizer";
    }

    public String getAlgorithmName(){
    	return "IVectorCategorizer";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
