package infomat.algorithms.labeling;

import infomat.algorithms.grouping.RelativeClusterer;
import infomat.algorithms.grouping.NeedsIMatrix;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups, the first in the centroid.
 */
public class RelativeLabeler extends Labeler{

    private IMatrix matrix;
    private boolean row;

    public RelativeLabeler(IMatrix matrix, boolean row){
	this.matrix = matrix;
	this.row=row;
	this.properties = getDefaultProperties();
    }

    public RelativeLabeler(IMatrix matrix, boolean row, mro.util.properties.Properties properties){
	this.matrix = matrix;
	this.row=row;
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Relative Labeler Properties", CurrentClassGetter.getClassName());
		
	try{


	    Property p = new Property("Number of objects in labels", 10, "The number of objects in each label");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    //    public static int numberOfObjects=10;

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /*********************************/



    private ILabel labelSingleGroup(IObjectGroup group, 
				    IObjectGroup relGroup){
	ILabel label = new ILabel(group);

	Vector<IObject> ios = relGroup.getOrdered();
	HashMap<IObject, Float> iofmap = relGroup.getObjectOrderValueMap();
	int numberOfObjects = (Integer) properties.getPropertyValue("Number of objects in labels");
	for(int i=0; i<numberOfObjects && i<ios.size(); i++){
	    IObject io = ios.elementAt(i);
	    float w = iofmap.get(io);
	    label.add(io, w);
	}
	return label;
    }

    /**
     * Constructs ILables for all IObjectGroup:s in the IObjectGrouping by
     * choosing the higest ranked IObject:s in the centroids.
     */
    public HashMap<IObjectGroup, ILabel> labelGrouping(IObjectGrouping ioging, SparseISimilarity similarity){

	if(!(similarity instanceof NeedsIMatrix)){
	    System.out.println("RelativeLabeler needs a SparseISimilarity that implements NeedsIMatrix!!!");
	    System.exit(0);
	}


	//	IMatrix matrix = similarity.getIMatrix();
	RelativeClusterer rl = new RelativeClusterer();
	rl.setSimilarity(similarity);
	
	IObjectSet iogingSet = matrix.getColumnIObjectSet();
	IObjectSet otherSet = matrix.getRowIObjectSet();
	//	boolean row = similarity.getIMatrixDimension();
	if(row){
	    iogingSet = matrix.getRowIObjectSet();
	    otherSet = matrix.getColumnIObjectSet();
	}
	iogingSet.setGrouping(ioging);

	IObjectGrouping relIOGING = rl.cluster(otherSet);
	


	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	    new HashMap<IObjectGroup, ILabel>();


	Vector<IObjectGroup> groups = ioging.getGroups();
	Vector<IObjectGroup> relGroups = relIOGING.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup group = groups.elementAt(i);
	    IObjectGroup relGroup = relGroups.elementAt(i);

	    //	    System.out.println("------ group: "+iog.getName());

	    ILabel label = labelSingleGroup(group, relGroup);
	    groupLabelMap.put(group, label);
	}

	return groupLabelMap;
    }


    /*********************************/
   

}

