package tctbridge;

import tctbridge.algorithms.grouping.*;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.text.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.measures.*;
import mro.util.properties.*;

import java.io.*;
import java.util.*;


/**
 * Writes the default Properties of the TCT-classes to a directory.
 * Good as a start for setting up experiments. Check that all classes
 * you need are included.
 * <br>
 *
 * @see infomat.WriteProperties
 *
 * @version          080415
 * @author           Magnus Rosell
 *
 *
 * javac -deprecation -classpath ./classes/:lib/tct-core.jar/:lib/tct-matrix.jar/:lib/tct-text.jar/: -sourcepath ./src ./src/tctbridge/*.java -d classes
 *
 *
 * java -cp .:classes/:lib/tct-core.jar:lib/tct-matrix.jar:lib/tct-text.jar -Xms1024m -Xmx1024m tctbridge.WriteTCTProperties -resultPath "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/AllPropertiesWritten/"
 */
public class WriteTCTProperties{
    private static void badInput(){
	System.out.println("Bad input!");
	System.exit(0);
    }


    public static void main(String args[]) throws IOException{

	if(args.length==0)
	    badInput();
	String resultPath=null;

	for(int i=0; i<args.length; i++){
	    if(args[i].equals("-resultPath"))
		resultPath=args[i+1];
	}
	if(resultPath==null)
	    badInput();

	if(!resultPath.endsWith(File.separator))
	    resultPath += File.separator;

	/**********************************/
	TCTKMeans tctKmeans = new TCTKMeans();
	tctKmeans.getProperties().saveToDirectory(resultPath);

	TCTAgglomerative tctAgglomerative = new TCTAgglomerative();
	tctAgglomerative.getProperties().saveToDirectory(resultPath);

	TCTBisectingKMeans tctBisectingKMeans = new TCTBisectingKMeans();
	tctBisectingKMeans.getProperties().saveToDirectory(resultPath);

    }
}
