package tctbridge.algorithms.grouping;

// Infomat
import tctbridge.io.*;
import infomat.algorithms.grouping.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import mro.util.*;
import mro.util.properties.*;

// TCT
import text.data.Corpus;
import core.metrics.similarity.CosineSimilarity;
import core.clustering.hierarchical.BisectingKMeans;
import core.clustering.hierarchical.BisectingKMeans.ClusterSelection;
import core.clustering.*;
import core.clustering.init.*;
import core.clustering.util.*;


// Java
import java.util.*;


/**
 * TCT Bisecting KMeans. Using the TCT package.
 *      
 * @version          080310
 * @author           Magnus Rosell
 */
public class TCTBisectingKMeans extends GroupingAlgorithm 
    implements NeedsIMatrix{ 

    private static int tctBisectingKMeansNumber = 1;


    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("TCT Bisecting-KMeans Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    Property p = new Property("Number of clusters", 5, "Number of clusers");
	    properties.addProperty(p);

	    p = new Property("Worst cluster", 1, "Definition of worst cluster");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "Average Similarity");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Size");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Number of Trials", 20, "Number of trials per bisection");
	    properties.addProperty(p);

	    /***************/


	    PropertyGroup pg = new PropertyGroup("K-Means Properties for Bisecting K-Means");

	    p = new Property("Init Type", 0, "Kind of initialization");
	    pvd = new PropertyValueDescription("0", "Random clustering as start clusters.");
	    p.addValueDescription(pvd);
	    pg.addProperty(p);

	    p = new Property("Maximum number of iterations", 500, "Maximum number of iterations");
	    pg.addProperty(p);
	    properties.addProperty(pg);

	    /********************/

	    p = new Property("Similarity Sort", true, "Sort objects in clusters according to similarity to cluster centroid.");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }
	
    private IMatrix matrix;

    public TCTBisectingKMeans(){
	this.properties = getDefaultProperties();
    }

    public TCTBisectingKMeans(mro.util.properties.Properties properties){
	this.properties=properties;
    }
	
    public void setIMatrix(IMatrix matrix){
	this.matrix=matrix;
    }

    public IMatrix getIMatrix(){
	return this.matrix;
    }



    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}

	return grouping;
    }



    public IObjectGrouping cluster(IObjectGroup group){
	TCTBridge tctB = new TCTBridge();
	Corpus corpus = tctB.getCorpus(matrix, group);

	int k = (Integer) properties.getPropertyValue("Number of clusters");
	CosineSimilarity metric = new CosineSimilarity();

	BisectingKMeans.ClusterSelection clusterSelection = 
	    BisectingKMeans.ClusterSelection.AVGSIM;
	int worstClusterType = (Integer) 
	    properties.getPropertyValue("Worst cluster");
	if(worstClusterType==0)
	    clusterSelection = 
		BisectingKMeans.ClusterSelection.AVGSIM;
	else if(worstClusterType==1)
	    clusterSelection = 
		BisectingKMeans.ClusterSelection.SIZE;

	BisectingKMeans clusterer = 
	    new BisectingKMeans(metric, k, clusterSelection);

	int trials = 
	    (Integer) properties.getPropertyValue("Number of Trials");
	clusterer.setTrials(trials);

	int initInt = (Integer) properties.getPropertyValue("Init Type");
	if(initInt==0)
	    clusterer.setInitStrategy(new InitRandomClusters());
	int maxIterations = (Integer) properties.getPropertyValue("Maximum number of iterations");
	clusterer.setMaxIterations(maxIterations);

	HardClustering clustering = null;
	try{
	    clustering = clusterer.findClusters(corpus);
	}catch(ClusteringException cle){
	    System.out.println(cle);
	    cle.printStackTrace();
	    System.exit(0);
	}

	boolean simOrder = (Boolean) 
	    properties.getPropertyValue("Similarity Sort");
	/*
	IObjectGrouping grouping = 
	    tctB.getIObjectGrouping(matrix, group, corpus, clustering, metric, simOrder, clusterer);
	*/

	IObjectGrouping tctGrouping = 
	    tctB.getIObjectGrouping(matrix, group, corpus, clustering, metric, simOrder, clusterer);
	
	String name = tctGrouping.getName()+tctBisectingKMeansNumber;
	IObjectGrouping grouping = new IObjectGrouping(name);
	/*
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}
	*/

	Vector<IObjectGroup> groups = tctGrouping.getGroups();
	for(int g=0; g<groups.size(); g++){
	    IObjectGroup iog = groups.elementAt(g);
	    grouping.add(iog);
	    //	    printGroup(group);
	}

	tctBisectingKMeansNumber++;



	return grouping;
    }

    public String getInfoString(){
    	return "TCTBisectingKMeans";
    }

    public String getAlgorithmName(){
    	return "TCTBisectingKMeans";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
