package infomat.io;

import infomat.vectorspace.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

/**
 * For reading a set of texts from a directory (and its subdirectories).
 * 
 * @version          080211
 * @author           Magnus Rosell
 */
public class TextDirectoryReader{

    private IMatrix matrix;
    private TextWordIMatrixConstructor twmc;
    private static int idCounter=1;

    public static TextFileFilter tff = new TextFileFilter();
    public static AllFileFilter aff = new AllFileFilter();

    public TextDirectoryReader(){
	this.twmc = new TextWordIMatrixConstructor();
    }

    public TextDirectoryReader(TextWordIMatrixConstructor twmc){
	this.twmc = twmc;
    }

    /**
     * Loads a IMatrix from a directory.
     */
    public IMatrix load(File file) {
	//	matrix = new IMatrix("Words", "Texts");
	    
	recurseFile(file);

	IMatrix m = twmc.getIMatrix();

	m.getRowIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	m.getColumnIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	
	m.getRowIObjectSet().makeStartGrouping();
	m.getColumnIObjectSet().makeStartGrouping();

	return m;
    }

    private void recurseFile(File file) {
	if(file.isDirectory()){
	    File[] list = file.listFiles(aff);
	    for(int i=0; i<list.length; i++){
		recurseFile(list[i]);
	    }
	}else{
	    doOneFile(file);
	}
    }

    private HashMap<String, IObject> stringWordObjectMap = 
	new HashMap<String, IObject>();

    private void doOneFile(File file){

	try {

	    int totalLocalCount = 0;
	    HashMap<String, Integer> stringNumberMap = 
		new HashMap<String, Integer>();


	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    String s=in.readLine();
	    while(s!=null){
		String[] words = s.split(" ");
		for(int i=0; i<words.length; i++){
		    
		    Integer number = stringNumberMap.get(words[i]);
		    if(number==null)
			number=0;
		    number++;
		    stringNumberMap.put(words[i], number);
		    totalLocalCount++;

		    //		    twmc.addWord(words[i], null, 1, 1, 1);

		}

		s=in.readLine();
	    }


	    twmc.addText(file.getName(), file.getPath());

	    Iterator<String> it = stringNumberMap.keySet().iterator();
	    while(it.hasNext()){
		String word = it.next();
		Integer n = stringNumberMap.get(word);
		twmc.addWord(word, null, n, ((float) n)/totalLocalCount, ((float) n)/totalLocalCount);
	    }

	}catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }
}


class TextFileFilter implements FileFilter{

    public boolean accept(File pathname){
	if(pathname.isDirectory())
	    return true;
	if(pathname.getName().endsWith(".txt") ||
	   pathname.getName().endsWith(".TXT"))
	    return true;
	return false;
    }

}

class AllFileFilter implements FileFilter{

    public boolean accept(File pathname){
	return true;
    }

}
