package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.bridges.tct.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/


// java -cp .:classes/ -Xms1024m -Xmx1024m infomat.LabelExamples


/**
 *
 * @version          080402
 * @author           Magnus Rosell
 *
 */
public class LabelExamples{

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	// This one has texts as rows and words as columns
	String inpath = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/20news-bydate-test/";
	//	String inpath = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/mini_newsgroups_processed/";
	File file = new File(inpath);
	TextDirectoryReader tdr = new TextDirectoryReader();
	IMatrix matrix = tdr.load(file);

	IObjectGrouping rowGrouping = 
	    matrix.getColumnIObjectSet().makeStartGrouping();
	IObjectGrouping colGrouping = 
	    matrix.getRowIObjectSet().makeStartGrouping();
	rowGrouping.sort();
	colGrouping.sort();


	// Filter the matrix
	// The example is to small
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(matrix);

	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(matrix);

	// Categorization
	LocationGrouper locationGrouper = new LocationGrouper();
	IObjectGrouping categorization = 
	    locationGrouper.cluster(matrix.getRowIObjectSet());

	// Cluster the rows (texts) of the matrix
	// Similarity
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);

	IObjectSet rowSet = matrix.getRowIObjectSet();
	KMeans kmeans = new KMeans(dotSim);
	IObjectGrouping clustering = kmeans.cluster(rowSet);
	clustering.removeEmptyGroups();

	clustering = categorization;

	String fileName = "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/labels/labels.tex";
	PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
	LatexHelper.printLatexHead(pw);

	/*** Labeling ***/

	CentroidLabeler cl = new CentroidLabeler();

	CentroidRankingBoostLabeler crbl = 
	    new CentroidRankingBoostLabeler();

	CombinedCentroidLabeler ccl = new CombinedCentroidLabeler();
	
	try{
	    cl.setProperty("Number of objects in labels", 20);
	    crbl.setProperty("Number of objects in labels", 20);
	    ccl.setProperty("Number of objects in labels", 20);
	}catch(PropertyNotFoundException pnfe){
	    System.out.println("oneReductionExperiment(): Number of objects in labels");
	    System.exit(0);
	}


	HashMap<IObjectGroup, ILabel> clLabeling = 
	    cl.labelGrouping(clustering, dotSim); 

	HashMap<IObjectGroup, ILabel> crblLabeling = 
	    crbl.labelGrouping(clustering, dotSim); 

	HashMap<IObjectGroup, ILabel> cclLabeling = 
	    ccl.labelGrouping(clustering, dotSim); 

	
	/*
	LabelReducer labelReducer = new LabelReducer();
	IObjectGrouping[] labelIOGINGs = labelReducer.reduceGroupingByLabeling(clustering, crblLabeling, matrix);
	IObjectGrouping labelReducedClustering = labelIOGINGs[0];
	labelReducedClustering.printGroupSizes();

	IObjectGrouping labelReducedClustering2 = labelIOGINGs[1];
	labelReducedClustering2.printGroupSizes();

	HashMap<IObjectGroup, ILabel> crblclLabeling = 
	    cl.labelGrouping(labelReducedClustering, dotSim); 

	Vector<IObjectGroup> reducedGroups = 
	    labelReducedClustering.getGroups();
	*/

	Vector<IObjectGroup> groups = clustering.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup g = groups.elementAt(i);
	    ILabel clLabel = clLabeling.get(g);
	    ILabel crblLabel = crblLabeling.get(g);
	    ILabel cclLabel = cclLabeling.get(g);

	    /*
	    IObjectGroup rg = reducedGroups.elementAt(i);
	    ILabel crblclLabel = crblclLabeling.get(rg);
	    */

	    Vector<ILabel> labels = new Vector<ILabel>();
	    labels.add(clLabel);
	    labels.add(crblLabel);
	    //	    labels.add(crblclLabel);
	    labels.add(cclLabel);
	    printTableTable(pw, g, labels);

	}

	LatexHelper.printLatexEnd(pw);
	pw.close();

    }

    private static void printTableTable(PrintWriter pw, IObjectGroup group, 
				 Vector<ILabel> labels) throws IOException{

	int numberOfColumns = labels.size()*2+1;
	LatexHelper.printTableHead(pw, numberOfColumns);
	
	ILabel il = labels.elementAt(0);
	int size = il.numberOfIObjects();

	for(int i=0; i<size; i++){
	    Vector<String> values = new Vector<String>();
	    for(int j=0; j<labels.size(); j++){
		il = labels.elementAt(j);
		if(il!=null){
		    IObject io = il.getOrdered().elementAt(i);
		    values.add(io.getString());
		    HashMap<IObject, Float> map = il.getObjectOrderValueMap();
		    values.add(""+Round.roundDecimalsToTrimmedString(map.get(io), 2));
		}else{
		    values.add(" ");
		    values.add(" ");
		}		    
	    }
	    LatexHelper.printTableRow(pw, values);
	}
	
	
	String caption = group.getName(); 
	String label = group.getName();
	LatexHelper.printTableEnd(pw, caption, label);

    }


    /*

    public static void label(IMatrix matrix, IObjectGrouping clustering, 
			     Labeler labeler, IObjectGrouping categorization,
			     ISimilarity similarity){

	String resPathName = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/labels/";

	// The current time
	long time = System.currentTimeMillis();
	

	String labelerName = labeler.getClass().getName();

	// Write labels to file
	try{
	    PrintWriter pw = 
		new PrintWriter(resPathName+time+"."+labelerName+".txt");
	    Labeler.printLabeling(pw, clustering, labeling, "K-Means + "+labelerName);
	    pw.close();
	}catch(IOException io){
	    // do nothing
	}

	// Evaluate and write the results
	String evaluationFileName = 
	    resPathName+time+"."+labelerName+".Evaluation.xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, categorization, similarity, ""+time);
	calculateAndSaveEvaluation(eval, evaluationFileName);
	
	System.out.println("Clustered!!!");
    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }

    */

}
