package infomat.mro.synlex;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import java.io.*;
import java.util.*;




/****************/



/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ -Xms1024m -Xmx1024m infomat.mro.synlex.SynlexExample</code>
 *
 * @version          
 * @author           Magnus Rosell
 *
 */


/*

 - Grupper inom vilka ord bara har n steg mellan sig.

 - Definiera avstnd som 1/level. Stigs avstnd summan eller produkten
   av avstnden. -> Grupper inom vilka ord ligger p hgst ett avstnd.

*/




public class SynlexExample{

    public static void main(String[] args) throws IOException{

	String synlexFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/synpairs.xml";
	//	String synlexFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/smallsynpairs.xml";
	//	String synlexFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/verysmallsynpairs.xml";
	File synlexFile = new File(synlexFileName);
	SynlexIO sio = new SynlexIO();
	IMatrix matrix = sio.load(synlexFile);

	System.out.println("matrix: "+matrix.getSize());

	float minWeight = 5.0f;

	SynlexGrouper synlexGrouper = new SynlexGrouper();
	//	IObjectGrouping grouping = synlexGrouper.getConnectedComponentsGrouping(matrix, minWeight);
	IObjectGrouping grouping = synlexGrouper.getClicksGrouping(matrix, minWeight);
	//	grouping.printGroupNamesAndSizes();
	System.out.println();
	System.out.println("number of objects: "+grouping.numberOfIObjects());
	System.out.println("number of groups: "+grouping.numberOfIObjectGroups());



	int castWeight = (int) minWeight;
	String fileName = "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/synlex/";
	//	fileName = fileName + "stat"+castWeight+".m";
	fileName = fileName + "clickStat"+castWeight+".m";
	writeMatlabVector(grouping, minWeight, fileName);

    }

    private static void writeMatlabVector(IObjectGrouping grouping, float minWeight, String fileName) throws IOException{

	System.out.println("Building size map...");

	int numberOfObjects=0;
	int largestSize = 0;
	HashMap<Integer, Integer> sizeNumberMap = 
	    new HashMap<Integer, Integer>();

	Vector<IObjectGroup> groups = grouping.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup group = groups.get(i);
	    int size = group.numberOfIObjects();

	    //	    System.out.println("Storlek: "+size);

	    if(size > largestSize)
		largestSize=size;
	    numberOfObjects += size;

	    Integer n = sizeNumberMap.get(size);
	    if(n==null)
		sizeNumberMap.put(size, 1);
	    else
		sizeNumberMap.put(size, n+1);
	}	    

	System.out.println("Writing to file...");

	
	File file = new File(fileName);
	PrintWriter out =
	    new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	out.print("sizevec = [");

	int secondNumberOfObjects = 0;
	for(int i=1; i<largestSize; i++){
	    //	    System.out.print(i+": ");
	    Integer s = sizeNumberMap.get(i);
	    if(s==null){
		out.print("0, ");
		//		System.out.print("0, ");
	    }else{
		out.print(s+", ");
		//		System.out.print(s+", ");
		secondNumberOfObjects += i*s;
	    }
	} 

	Integer s = sizeNumberMap.get(largestSize);
	out.println(s+"];");
	//	System.out.println(s+"];");
	secondNumberOfObjects += largestSize*s;

	out.flush();
	out.close();

	System.out.println();
	System.out.println("* Number Of Objects: "+numberOfObjects);
	System.out.println("* Second Number Of Objects: "+secondNumberOfObjects);
	//	System.out.println("* Number Of Objects in Set: "+setSetMap.size());
	System.out.println();
	

    }


    /*
    private static void writeMatlabVector(HashSet<HashSet<IObject>> setSet, float minWeight) throws IOException{

	System.out.println("Building size map...");

	int numberOfObjects=0;
	int largestSize = 0;
	HashMap<Integer, Integer> sizeNumberMap = 
	    new HashMap<Integer, Integer>();

	Iterator<HashSet<IObject>> it = setSet.iterator();
	while(it.hasNext()){
	    HashSet<IObject> set = it.next();
	    int size = set.size();
	    if(size > largestSize)
		largestSize=size;
	    //	    System.out.println("Size of set: "+size);
	    numberOfObjects += size;

	    Integer i = sizeNumberMap.get(set.size());
	    if(i==null)
		sizeNumberMap.put(set.size(), 1);
	    else
		sizeNumberMap.put(set.size(), i+1);
	}	    

	System.out.println("Writing to file...");

	int castWeight = (int) minWeight;
	
	File file = new File("/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/synlex/stat"+castWeight+".m");
	PrintWriter out =
	    new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	out.print("sizevec = [");
	
	for(int i=1; i<largestSize-1; i++){
	    //	    System.out.print(i+", ");
	    Integer s = sizeNumberMap.get(i);
	    if(s==null)
		out.print("0, ");
	    else
		out.print(s+", ");
	} 

	Integer s = sizeNumberMap.get(largestSize);
	out.println(s+"];");

	out.flush();
	out.close();

	System.out.println();
	System.out.println("* Number Of Objects: "+numberOfObjects);
	//	System.out.println("* Number Of Objects in Set: "+setSetMap.size());
	System.out.println();
	

    }

    */



}
