package infomat.mro.synlex;

import infomat.vectorspace.*;

import java.io.*;
import java.util.*;


/****************/



/**
 *
 *
 * @version          
 * @author           Magnus Rosell
 *
 */
public class SynlexGrouper{


    public IObjectGrouping getClicksGrouping(IMatrix matrix, float minWeight){

	HashMap<IObject, HashSet<IObject>> ioSetMap = 
	    getClicksHashSets(matrix, minWeight);

	HashSet<HashSet<IObject>> setSet = getSetSetFromIOSetMap(ioSetMap);

	return getGroupingFromSetSet(setSet, minWeight);
    }


    public IObjectGrouping getConnectedComponentsGrouping(IMatrix matrix, float minWeight){

	HashMap<IObject, HashSet<IObject>> ioSetMap = 
	    getConnectedComponentsHashSets(matrix, minWeight);

	HashSet<HashSet<IObject>> setSet = getSetSetFromIOSetMap(ioSetMap);
	
	return getGroupingFromSetSet(setSet, minWeight);
    }


    private IObjectGrouping getGroupingFromSetSet(HashSet<HashSet<IObject>> setSet, float minWeight){

	System.out.println("Creating grouping...");

	IObjectGrouping grouping = new IObjectGrouping("Synlex Grouping level="+minWeight);


	int groupNr = 1;
	Iterator<HashSet<IObject>> it = setSet.iterator();
	while(it.hasNext()){

	    HashSet<IObject> set = it.next();

	    if(groupNr % 100 == 0 || groupNr == 1){
		System.out.println("Group "+groupNr);
		System.out.println("Size of set: "+set.size());
	    }

	    IObjectGroup group = new IObjectGroup("Group "+groupNr);
	    group.addAll(new Vector<IObject>(set));

	    if(groupNr % 100 == 0 || groupNr == 1)
		System.out.println("   Adderat IObjects");

	    grouping.add(group);

	    if(groupNr % 100 == 0 || groupNr == 1)
		System.out.println("   Adderat Group");

	    groupNr++;
	}

	return grouping;

    }



    /*
    public HashSet<HashSet<IObject>> getConnectedComponents(IMatrix matrix, float minWeight){
	
	HashSet<HashSet<IObject>> setSet = new HashSet<HashSet<IObject>>();

	Iterator<HashSet<IObject>> it = ioSetMap.values().iterator();
	while(it.hasNext()){
	    HashSet<IObject> thisSet = it.next();
	    //	    printSet(thisSet);
	    //	    System.out.println(thisSet+": "+thisSet.size());
	    setSet.add(thisSet);
	}

	return setSet;
    }
    */

    private HashSet<HashSet<IObject>> getSetSetFromIOSetMap(HashMap<IObject, HashSet<IObject>> ioSetMap){

	HashSet<HashSet<IObject>> setSet = new HashSet<HashSet<IObject>>();

	Iterator<HashSet<IObject>> it = ioSetMap.values().iterator();
	while(it.hasNext()){
	    HashSet<IObject> thisSet = it.next();
	    setSet.add(thisSet);
	}

	return setSet;
    }



    private void printSet(HashSet<IObject> set){
	System.out.print("Set: [");
	Iterator<IObject> it = set.iterator();
	while(it.hasNext()){
	    IObject io = it.next();
	    System.out.print(io.getString()+", ");
	}
	System.out.println("]");
    }



    private HashMap<IObject, HashSet<IObject>> 
	getConnectedComponentsHashSets(IMatrix matrix, float minWeight){
	
	int sizeOfLargestSet = 1;
	HashMap<IObject, HashSet<IObject>> ioSetMap = 
	    new HashMap<IObject, HashSet<IObject>>();

	Vector cells = matrix.getAllCells();
	
	//	System.out.println("Number of cells: "+cells.size());


	for(int i=0; i<cells.size(); i++){

	    if(i % 100 == 0){
		//	    if(i % 1 == 0){
		//		grouping.printGroupNamesAndSizes();
		System.out.println("Cells: "+cells.size());
		System.out.println("i: "+i);
		//		System.out.println("Number of Sets: "+ioSetMap.size());
		System.out.println("Size of Largest Groups: "+sizeOfLargestSet);
	    }


	    IMatrixCell mc = (IMatrixCell) cells.get(i);

	    if(mc.getWeight() < minWeight)
		continue;

	    IObject io1 = mc.getRowIObject();
	    IObject io2 = mc.getColumnIObject();

	    //	    System.out.println("MatrixCell: "+mc.getRowIObject().getString()+", "+mc.getColumnIObject().getString());

	    HashSet<IObject> set1 = ioSetMap.get(io1);
	    if(set1==null){
		set1 = new HashSet<IObject>();
		set1.add(io1);
		ioSetMap.put(io1, set1);
	    }

	    if(io2!=io1){
		HashSet<IObject> set2 = ioSetMap.get(io2);
		if(set2==null){
		    set2 = new HashSet<IObject>();
		    set2.add(io2);
		    ioSetMap.put(io2, set2);
		}

		if(set1 != set2){

		    //		    System.out.println("************** REMOVING");
		    
		    // Remove
		    ioSetMap.remove(io1);
		    ioSetMap.remove(io2);
		    

		    
		    HashSet<IObject> set3 = new HashSet<IObject>();
		    set3.addAll(set1);		
		    set3.addAll(set2);
		    //		set1.addAll(set2);
		    
		    Iterator<IObject> it = set3.iterator();
		    while(it.hasNext()){
			ioSetMap.put(it.next(), set3);
		    }
		    //		    ioSetMap.put(io1, set3);
		    //		    ioSetMap.put(io2, set3);
		    
		    if(set1.size()> sizeOfLargestSet)
			sizeOfLargestSet = set1.size();
		    
		    //		    System.out.println("Ett nytt");
		    //		    printSet(set3);

		}

		/*
		else{
		    System.out.println("Tv nya");
		    printSet(set1);
		    printSet(set2);

		    
		}
		*/

	    }

	    /*
	    else{
		System.out.println("Inget nytt");

	    }

		System.out.println("----------------------");
	    */
	}

	return ioSetMap;
    }





    private HashMap<IObject, HashSet<IObject>> 
	getClicksHashSets(IMatrix matrix, float minWeight){
	
	int sizeOfLargestSet = 1;
	HashMap<IObject, HashSet<IObject>> ioSetMap = 
	    new HashMap<IObject, HashSet<IObject>>();

	Vector cells = matrix.getAllCells();
	
	for(int i=0; i<cells.size(); i++){

	    if(i % 100 == 0){
		//	    if(i % 1 == 0){
		//		grouping.printGroupNamesAndSizes();
		System.out.println("Cells: "+cells.size());
		System.out.println("i: "+i);
		//		System.out.println("Number of Sets: "+ioSetMap.size());
		System.out.println("Size of Largest Groups: "+sizeOfLargestSet);
	    }


	    IMatrixCell mc = (IMatrixCell) cells.get(i);

	    if(mc.getWeight() < minWeight)
		continue;

	    IObject io1 = mc.getRowIObject();
	    IObject io2 = mc.getColumnIObject();

	    //	    System.out.println("MatrixCell: "+mc.getRowIObject().getString()+", "+mc.getColumnIObject().getString());

	    HashSet<IObject> set1 = ioSetMap.get(io1);
	    if(set1==null){
		set1 = new HashSet<IObject>();
		set1.add(io1);
		ioSetMap.put(io1, set1);
	    }

	    if(io2 != io1){
		HashSet<IObject> set2 = ioSetMap.get(io2);
		if(set2==null){
		    set2 = new HashSet<IObject>();
		    set2.add(io2);
		    ioSetMap.put(io2, set2);
		}

		if(set1 != set2){

		    // Check if click!
		    if(!isCombinationClick(matrix, minWeight, set1, set2))
			continue;

		    
		    // Remove
		    ioSetMap.remove(io1);
		    ioSetMap.remove(io2);
		    

		    
		    HashSet<IObject> set3 = new HashSet<IObject>();
		    set3.addAll(set1);		
		    set3.addAll(set2);
		    //		set1.addAll(set2);
		    
		    Iterator<IObject> it = set3.iterator();
		    while(it.hasNext()){
			ioSetMap.put(it.next(), set3);
		    }
		    //		    ioSetMap.put(io1, set3);
		    //		    ioSetMap.put(io2, set3);
		    
		    if(set1.size()> sizeOfLargestSet)
			sizeOfLargestSet = set1.size();
		    
		    //		    System.out.println("Ett nytt");
		    //		    printSet(set3);

		}

		/*
		else{
		    System.out.println("Tv nya");
		    printSet(set1);
		    printSet(set2);

		    
		}
		*/

	    }

	    /*
	    else{
		System.out.println("Inget nytt");

	    }

		System.out.println("----------------------");
	    */
	}

	return ioSetMap;
    }

    private boolean isCombinationClick(IMatrix matrix, 
				       float minWeight,
				       HashSet<IObject> set1, 
				       HashSet<IObject> set2){

	Iterator<IObject> it1 = set1.iterator();
	while(it1.hasNext()){
	    IObject o1 = it1.next();
	    Iterator<IObject> it2 = set2.iterator();
	    while(it2.hasNext()){
		IObject o2 = it2.next();
		IMatrixCell mc = matrix.getCell(o1, o2);
		if(mc==null)
		    mc = matrix.getCell(o2, o1);
		if(mc==null || mc.getWeight() < minWeight)
		    return false;
	    }
	}
	return true;
    }






}
