package infomat.vectorspace;

import infomat.vectorspace.vector.IVectorIndex;

import mro.util.CommonPath;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

/**
 * A IObject is either an object or a feature.
 * It could be a word or a document for instance.
 * This is the smallest part of the representation.
 *
 * @param s  A String describing the IObject
 * @param locationPath A String locating the object in the filesystem (the path)
 * @param location A String locating the object in the filesystem (the file name)
 * @param kind  The kind of IObject. 0 - IObject, 1 - IObjectGroup, 2 - IObjectGrouping, 3 - IObjectSet
 * @param id  Index, a unique identifier for the IObject
 *
 * @version          070314
 * @author           Magnus Rosell
 */
public class IObject implements IVectorIndex{
    private static CommonPath commonPath = new CommonPath();

    private String s=null;
    private String locationPath=null;
    private String location=null;
    private int kind;
    private int id=0;
    private boolean ambiguous=false;

    public IObject(String s, int id){
	this.s=s;
	this.kind=0;
	this.id=id;
    }

    public IObject(String s, String location, int id){
	this.s=s;

	Vector<String> locs = commonPath.getPathAndFileName(location);
	this.locationPath = locs.elementAt(0);
	this.location = locs.elementAt(1);

	this.kind=0;
	this.id=id;
    }

    public IObject(String s, int kind, int id){ 
	this.s=s;
	this.kind=kind;
	this.id=id;
    }

    public IObject(String s, String location, int kind, int id){
	this.s=s;

	Vector<String> locs = commonPath.getPathAndFileName(location);
	this.locationPath = locs.elementAt(0);
	this.location = locs.elementAt(1);

	this.kind=kind;
	this.id=id;
    }


    public String getString(){
	if(s==null)
	    return "id("+id+")";
	if(ambiguous)
	    return s+":"+id;
	return s;
    }

    public void setString(String s){
	this.s=s;
    }

    public void setAmbiguous(boolean ambiguous){
	this.ambiguous=ambiguous;
    }

    public boolean getAmbiguous(){
	return this.ambiguous;
    }

    public String getAmbiguousString(){
	return s;
    }

    public String getLocation(){
	if(location!=null){
	    if(locationPath!=null)
		return locationPath+location;
	    return location;
	}
	return null;
    }

    public int getKind(){
	return kind;
    }

    public int getID(){
	return id;
    }


    /**
     * Returns the representation in the form of a HashMap, mapping
     * IObject to a Float value. For a group - the centroid.
     */
    /*
    public HashMap<IObject, Float> getRepresentation(IMatrix myIMatrix){
	HashMap<IObject, Float> centroid = new HashMap<IObject, Float>();

	Vector features = myIMatrix.getCellsWith(this);
	if(features!=null){
	    for(int f=0; f<features.size(); f++){
		IMatrixCell sc = (IMatrixCell) features.elementAt(f);
		IObject feature = sc.getRowIObject();
		if(this==feature)
		    feature = sc.getColumnIObject();
		centroid.put(feature, sc.getWeight());
	    }
	}
	return centroid;
    }
    */

    public static String getLargestCommonLocation(){
	return commonPath.getLargestCommonPath();
    }


    public int stringCompareTo(IObject co){
	return getString().compareTo(co.getString());
    }
}

