package infomat.vectorspace.weighting;

import infomat.vectorspace.*;

import mro.util.*;
import mro.util.properties.*;

import java.lang.Math.*;
import java.util.*;

/**
 * A class for weighting of a IMatrix.
 *
 * Currently works in a specific direction! Considers rows to be objects
 * (texts) and columns to be features (words).
 *
 * @version          080619
 * @author           Magnus Rosell
 */
public class TFIDFWeighter extends Weighter{

    public TFIDFWeighter(){
	super();
    }

    public TFIDFWeighter(mro.util.properties.Properties properties){
	super(properties);
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("TFIDF Weighter Properties", CurrentClassGetter.getClassName());
		
	try{

	    PropertyGroup pg = new PropertyGroup("Local Row and Global Column Weight");
	    
	    // Local row weight
	    Property p = new Property("Local row weight", 3, "The kind of local weight for elements in their row");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "No weight.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Current local weight.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("2", "Simple relative frequency.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("3", "tf according to Croft 1983.");
	    p.addValueDescription(pvd);
	    pg.addProperty(p);

	    p = new Property("(Local row/global column) weight importance factor", 0.5, "Used in Croft 1983. Between 0 (local important) and 1 (global important).");
	    pg.addProperty(p);

	    // Global column weight
	    p = new Property("Global column weight", 3, "The kind of global weight for column objects");
	    pvd = new PropertyValueDescription("0", "No weight.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Current global weight.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("2", "Simple log-idf.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("3", "idf according to Croft and Harper 1979.");
	    p.addValueDescription(pvd);
	    pg.addProperty(p);

	    p = new Property("Global column weight belief factor", 0.0, "Used in Croft 1983. Between 0 (automatically indexed, full belief in global weight) and 1 (manually indexed, full belief in manual indexer).");
	    pg.addProperty(p);

	    properties.addProperty(pg);

	    // Normalization
	    pg = new PropertyGroup("Normalization");
	    p = new Property("Row normalization", true, "Wether the rows are normalized after the weighting");
	    pg.addProperty(p);
	    properties.addProperty(pg);





	    // Local column weight
	    /*
	      p = new Property("Local column weight", 0, "The kind of local weight for elements in their column");
	      pvd = new PropertyValueDescription("0", "No weight.");
	      p.addValueDescription(pvd);
	      pvd = new PropertyValueDescription("1", "Current local weight.");
	      p.addValueDescription(pvd);
	      pvd = new PropertyValueDescription("2", "Simple relative frequency.");
	      p.addValueDescription(pvd);
	      pvd = new PropertyValueDescription("3", "tf according to Croft 1983.");
	      p.addValueDescription(pvd);
	      properties.addProperty(p);

	      p = new Property("Local column weight importance factor", 0.5, "Used in Croft 1983. Between 0 and 1.");
	      properties.addProperty(p);
	    */


	    /*
	      p = new Property("", -1, "");
	      properties.addProperty(p);
	    */

	    /*	    
		   p = new Property("Remove rows with more elements than", -1, "Rows with more elements are removed");
		   PropertyValueDescription pvd = new PropertyValueDescription("-1", "The total number of columns.");
		   p.addValueDescription(pvd);
		   pvd = new PropertyValueDescription("-2", "Half the total number of columns.");
		   p.addValueDescription(pvd);
		   properties.addProperty(p);
	    */

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }

    public String getWeighterName(){
	return "TFIDF-Weighter";
    }

    /*
      public static HashMap<IObject, Float> localWeightInGroup(IObjectGroup ig, boolean row, IMatrix matrix, mro.util.properties.Properties properties){ 
	
      //
      // Think of all as a document-term-matrix.
      // Calculating the global weight for the terms (columns).
      //
      }
    */



    /**
     * Weight the IMatrix (the whole matrix, 
     * which may be more than the current IObjectGroupings, the picture).
     */
    public void weight(IMatrix matrix){ 
	int localRowWeight = (Integer) properties.getPropertyValue("Local row weight");
	boolean anyLocalRowWeight=true;
	if(localRowWeight==0)
	    anyLocalRowWeight=false;

	//Local weight
	boolean row = true;
	IObjectGroup objectGroup = matrix.getRowIObjectSet();
	IObjectGroup representationGroup = matrix.getColumnIObjectSet();
	HashMap<IObject, HashMap<IObject, Float>> mapMap = localWeightInGroupByGroup(objectGroup, representationGroup, matrix, row, properties);

	//	System.out.println("******* weight");
	//	System.out.println("mapMap: "+mapMap.size());
	if(mapMap!=null){
	    Vector<IObject> rowObjects =
		matrix.getRowIObjectSet().getOrdered();
	    for(int i=0; i<rowObjects.size(); i++){
		IObject rowObj = rowObjects.elementAt(i);
		HashMap<IObject, Float> map = mapMap.get(rowObj);
		//		System.out.println("map: "+map.size());
		/*
		Iterator<IObject> iter = map.keySet().iterator();
		while(iter.hasNext()){
		    System.out.println(" - "+iter.next());
		}
		*/

		HashSet cells = 
		    matrix.getColumnIMatrixCellsForRowObject(rowObj);
		//		System.out.println("cells: "+cells.size());
		if(cells==null)
		    continue;
		Iterator cellIt = cells.iterator();
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    //		    System.out.println(mc.getRowIObject() +", "+mc.getColumnIObject()+": "+mc.getWeight());
		    //		    System.out.println("other: "+mc.getOtherIObject(rowObj));
		    float localWeight = map.get(mc.getOtherIObject(rowObj));
		    mc.setLocalWeight(localWeight);
		    mc.setWeight(localWeight);
		}
	    }
	}


	    /*
	int localRowWeight = (Integer) properties.getPropertyValue("Local row weight");
	boolean anyLocalRowWeight=true;
	if(localRowWeight==0)
	    anyLocalRowWeight=false;

	// Local row weight
	// localRowWeight==1: use current local weight
	if(anyLocalRowWeight && localRowWeight!=1){ 
	    Vector<IObject> rowObjects =
		matrix.getRowIObjectSet().getOrdered();
	    for(int i=0; i<rowObjects.size(); i++){
		HashSet cells = 
		    matrix.getColumnIMatrixCellsForRowObject(rowObjects.elementAt(i));
		Iterator cellIt = cells.iterator();
		float total = 0f;
		float maxCount = 0f;
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    total += mc.getLocalCount();
		    maxCount = (float) Math.max(maxCount, mc.getLocalCount());
		}
		cellIt = cells.iterator();
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    float localWeight=0.0f;
		    if(localRowWeight==2){ // simple relative frequency
			localWeight = mc.getLocalCount()/total;
		    }else if(localRowWeight==3){ // Croft 1983
			double factor = (Double) properties.getPropertyValue("(Local row/global column) weight importance factor");
			localWeight = (float)
			    (factor + (1-factor)*mc.getLocalCount()/maxCount);
		    }
		    mc.setLocalWeight(localWeight);
		    mc.setWeight(localWeight);
		}		
	    }
	}
	    */

	// Global column weight
	int globalColumnWeight = (Integer) properties.getPropertyValue("Global column weight");
	boolean anyGlobalColumnWeight=true;
	if(globalColumnWeight==0)
	    anyGlobalColumnWeight=false;
	if(anyGlobalColumnWeight){

	    IObjectSet columnIObjects = matrix.getColumnIObjectSet();
	    row = false;
	    HashMap<IObject, Float> objectGlobalWeightMap = 
		//		globalWeightInGroup(columnIObjects, row, matrix, properties);
		globalWeightInGroupByGroup(columnIObjects, matrix.getRowIObjectSet(), matrix, row, properties);
	    
	    Iterator<Map.Entry<IObject, Float>> it = 
		objectGlobalWeightMap.entrySet().iterator();
	    while(it.hasNext()){
		Map.Entry<IObject, Float> entry = it.next();
		IObject colObj = entry.getKey();
		float globalWeight = entry.getValue();
		matrix.setGlobalWeight(colObj, globalWeight);

		HashSet cells = 
		    matrix.getRowIMatrixCellsForColumnObject(colObj);
		Iterator it2 = cells.iterator();
		while(it2.hasNext()){
		    IMatrixCell mc = (IMatrixCell) it2.next();
		    if(anyLocalRowWeight){
			mc.setWeight(mc.getLocalWeight()*globalWeight);
		    }else{
			mc.setWeight(globalWeight);
		    }
		}
	    }

	}



	boolean rowNormalization = (Boolean) properties.getPropertyValue("Row normalization");
	if(rowNormalization){
	    row = true;
	    normalizeMatrix(matrix, row);
	}

    }

    /**
     * Normalizes the rows or columns of the matrix.
     */
    public static void normalizeMatrix(IMatrix matrix, boolean row){
	Vector<IObject> rowObjects = 
	    matrix.getColumnIObjectSet().getOrdered();
	IObjectSet colSet = matrix.getRowIObjectSet();
	if(row){
	    rowObjects = matrix.getRowIObjectSet().getOrdered();
	    colSet = matrix.getColumnIObjectSet();
	}

	for(int i=0; i<rowObjects.size(); i++){
	    IObject rowObj = rowObjects.elementAt(i);
	    Vector cells = 
		matrix.getCellsWithIObjectRestrictedToGroup(rowObj, colSet);
	    if(cells==null)
		continue;
	    double sqsum = 0.0;
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		sqsum += Math.pow(mc.getWeight(),2);
	    }

	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		mc.setWeight((float) Math.sqrt(Math.pow(mc.getWeight(),2)/sqsum));
	    }
	}
    }
    


    /**
     * Returns the sum of the local weight as calculated in 
     * #localWeightInGroupByGroup.
     */
    public static HashMap<IObject, Float> totalLocalWeightInGroupByGroup(IObjectGroup objectGroup, IObjectGroup representationGroup, IMatrix matrix, boolean row, mro.util.properties.Properties properties){ 
	
	HashMap<IObject, Float> totalLocalWeightMap = new HashMap<IObject, Float>();

	HashMap<IObject, HashMap<IObject, Float>> mapMap = localWeightInGroupByGroup(objectGroup, representationGroup, matrix, row, properties);
	System.out.println("mapMap, size: "+mapMap.size());

	/*******/
	/*
	Iterator<Map.Entry<IObject, HashMap<IObject, Float>>> tempIt = 
	    mapMap.entrySet().iterator();
	while(tempIt.hasNext()){
	    Map.Entry<IObject, HashMap<IObject, Float>> entry = tempIt.next();
	    System.out.println("m: "+entry.getKey().getString());
	    HashMap<IObject, Float> tempMap = entry.getValue();
	    Iterator<Map.Entry<IObject, Float>> tempIt2 = 
		tempMap.entrySet().iterator();
	    while(tempIt2.hasNext()){
		Map.Entry<IObject, Float> entry2 = tempIt2.next();
		System.out.println(entry2.getKey().getString()+": "+entry2.getValue());
	    }
	}
	*/
	/*******/


	// Hr ngonstans. Antagligen vnt p rader och kolumner.

	Iterator<HashMap<IObject, Float>> it = mapMap.values().iterator();
	while(it.hasNext()){
	    HashMap<IObject, Float> weightMap = it.next();
	    //	    System.out.println("weightMap, size: "+weightMap.size());

	    Iterator<Map.Entry<IObject,Float>> it2 = 
		weightMap.entrySet().iterator();
	    while(it2.hasNext()){
		Map.Entry<IObject,Float> entry = it2.next();
		//		System.out.println(" -0 "+entry.getKey().getString()+", "+entry.getValue());
		Float sum = totalLocalWeightMap.get(entry.getKey());
		//		System.out.println(" -1 "+sum);
		if(sum!=null)
		    totalLocalWeightMap.put(entry.getKey(), 
					    sum+entry.getValue());
		else
		    totalLocalWeightMap.put(entry.getKey(), entry.getValue());
		//		System.out.println(" -2 "+totalLocalWeightMap.get(entry.getKey()));
	    }
	}
	return totalLocalWeightMap;
    }


    /**
     * The central local weighter. 
     * The elements of the first group are considered objects, and
     * those of the second their representation. The objects in the second
     * are weighted accoring to occurences for each object in the first.
     * For text-word: the words are weighted locally for each text.
     * OBS. Only works in row-col order yet!!!
     *
     * @param row Wether the first group is along the rows of the matrix.
     */
    public static HashMap<IObject, HashMap<IObject, Float>> localWeightInGroupByGroup(IObjectGroup objectGroup, IObjectGroup representationGroup, IMatrix matrix, boolean row, mro.util.properties.Properties properties){ 

	int localRowWeight = (Integer) properties.getPropertyValue("Local row weight");
	boolean anyLocalRowWeight=true;
	if(localRowWeight==0)
	    anyLocalRowWeight=false;

	// Local row weight
	// localRowWeight==1: use current local weight
	if(anyLocalRowWeight && localRowWeight!=1){ 

	    HashMap<IObject, HashMap<IObject, Float>> ret = 
		new HashMap<IObject, HashMap<IObject, Float>>();

	    //
	    // Think of all as a document-term-matrix.
	    // Calculating the local weight for the terms (columns) 
	    // in each document (row).
	    //
	    Vector<IObject> rowObjects = objectGroup.getOrdered();
	    Vector<IObject> colObjects = representationGroup.getOrdered();
	
	    // OBS. The weighting is not completely general, yet.
	    double localWeightImportanceFactor = 0.0; // (Double) properties.getPropertyValue("(Local column/global row) weight importance factor");
	    if(row){
		localWeightImportanceFactor = (Double) properties.getPropertyValue("(Local row/global column) weight importance factor");
	    }

	    for(int i=0; i<rowObjects.size(); i++){
		HashMap rowMap = new HashMap<IObject, Float>();

		IObject rowObj = rowObjects.elementAt(i);
		//	    Vector cells = matrix.getCellsWith(rowObj);
		Vector cells = matrix.getCellsWithIObjectRestrictedToGroup(rowObj, representationGroup);
		if(cells==null)
		    continue;
		Iterator cellIt = cells.iterator();
		float total = 0f;
		float maxCount = 0f;
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    total += mc.getLocalCount();
		    maxCount = (float) Math.max(maxCount, mc.getLocalCount());
		}
		cellIt = cells.iterator();
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    float localWeight=0.0f;
		    if(localRowWeight==2){ // simple relative frequency
			localWeight = mc.getLocalCount()/total;
		    }else if(localRowWeight==3){ // Croft 1983
			localWeight = (float)
			    (localWeightImportanceFactor + 
			     (1-localWeightImportanceFactor) * 
			     mc.getLocalCount()/maxCount);
		    }
		    rowMap.put(mc.getOtherIObject(rowObj), localWeight);
		    //		mc.setLocalWeight(localWeight);
		    //		mc.setWeight(localWeight);
		}		
		ret.put(rowObj, rowMap);

	    }
	    return ret;
	}
	return null;
    }


    /**
     * The central global weighter. Weights the elements of the first group 
     * by their occurences in the matrix corresponding to the second group.
     *
     * @param row Wether the first group is along the rows of the matrix.
     */
    public static HashMap<IObject, Float> globalWeightInGroupByGroup(IObjectGroup weightGroup, IObjectGroup correspondingGroup, IMatrix matrix, boolean row, mro.util.properties.Properties properties){ 

	//
	// Think of all as a document-term-matrix.
	// Calculating the global weight for the terms (columns).
	//
	Vector<IObject> colObjects = weightGroup.getOrdered();
	Vector<IObject> rowObjects = correspondingGroup.getOrdered();
	int numberOfRows = rowObjects.size();

	int globalWeightKind = (Integer) 
	    properties.getPropertyValue("Global column weight");
	double globalBeliefFactor = (Double) properties.getPropertyValue("Global column weight belief factor");
	if(row){
	    globalWeightKind = (Integer) 
		properties.getPropertyValue("Global row weight");
	    globalBeliefFactor = (Double) properties.getPropertyValue("Global row weight belief factor");
	}

	HashMap<IObject, Float> objectGlobalWeightMap = 
	    new HashMap<IObject, Float>();
	for(int i=0; i<colObjects.size(); i++){
	    IObject colObj = colObjects.elementAt(i);
	    //	    Vector cells = matrix.getCellsWith(colObj);
	    Vector cells = matrix.getCellsWithIObjectRestrictedToGroup(colObj, correspondingGroup);
	    if(cells==null)
		continue;
	    int occurInRows = cells.size();
	    float globalWeight = 0.0f;
	    if(globalWeightKind==1)
		globalWeight = matrix.getGlobalWeight(colObj);
	    else if(globalWeightKind==2){
		if(occurInRows<1)
		    globalWeight = Float.NEGATIVE_INFINITY;
		else
		    globalWeight=(float) 
			MoreMath.log2(((double) numberOfRows/occurInRows));
	    }else if(globalWeightKind==3){
		if(occurInRows<1 || numberOfRows<=occurInRows)
		    globalWeight = Float.NEGATIVE_INFINITY;
		else
		    globalWeight = (float) 
			(globalBeliefFactor + MoreMath.log2((((double) numberOfRows) - occurInRows)/occurInRows));
	    }


	    //	    System.out.println(colObj.getString()+", occ: "+occurInRows+", weight: "+globalWeight);

	    /*
	      if(globalWeightingScheme==5){ // idf Croft and Harper 1979
	      // C in Croft 1983 is set to 0.5
	      // weight = (C + IDF) * TF
	      int numOfDocs=corpus.documentGroup.size();
	      double log2=Math.log(2);
	      int occurrInCollection;
	      while(li.hasNext()){
	      wordImage=(WordImage) li.next();
	      occurrInCollection=wordImage.setOfWordLists().size();
	      float c = (float) 0.5;
	      wordImage.globalWeight=(c + (float)(Math.log((numOfDocs-occurrInCollection)/occurrInCollection)/log2));
	      }
	      }
	    */

		    
	    objectGlobalWeightMap.put(colObj, globalWeight);
	}
	return objectGlobalWeightMap;
    }


}
