package infomat.view.algorithms;

import infomat.algorithms.grouping.*;
import infomat.vectorspace.*;
import infomat.vectorspace.weighting.*;
import infomat.view.*;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;


/**
 * A JPanel for choosing which Clustering Algorithm to use. 
 *
 * @see MatrixWeightingFrame
 *
 * @version          080409
 * @author           Magnus Rosell
 */
public class MatrixWeightingSelectionPanel extends JPanel
    implements ActionListener{

    private MatrixWeightingFrame weighterFrame;
    private JComboBox cBox;


    private String selected;


    private static Vector<String> weighterNames = null;
    private static HashMap<String, Weighter> nameWeighterMap = null;
	
    private static void initWeighters(){
	if(weighterNames==null){
	    weighterNames = new Vector<String>();
	    nameWeighterMap = new HashMap<String, Weighter>();
	    
	    Weighter weighter = new TFIDFWeighter();
	    nameWeighterMap.put(weighter.getWeighterName(), weighter);
	    weighterNames.add(weighter.getWeighterName());

	    weighter = new TFWeighter();
	    nameWeighterMap.put(weighter.getWeighterName(), weighter);
	    weighterNames.add(weighter.getWeighterName());

	    weighter = new CountWeighter();
	    nameWeighterMap.put(weighter.getWeighterName(), weighter);
	    weighterNames.add(weighter.getWeighterName());
	}
    }



    public MatrixWeightingSelectionPanel(MatrixWeightingFrame weighterFrame){
	this.weighterFrame = weighterFrame;
	initWeighters();
	selected = weighterNames.elementAt(0);
	makeComboBox();
    }

    public boolean actionLeadToChange(){
	String newSelected = (String) cBox.getSelectedItem();
	//	System.out.println("newSelected: "+newSelected);
	if(selected != newSelected){
	    selected = newSelected;
	    return true;
	}
	return false;
    }

    public Weighter getSelectedWeighter(){
	return nameWeighterMap.get(selected);
    }


    public void paintComponent(Graphics g) {
	super.paintComponent(g);
    }



    public void makeComboBox(){
	removeAll();
	cBox = new JComboBox(weighterNames);
	cBox.setSelectedItem(selected);
	add(cBox);
	//	cBox.addActionListener(algFrame);
	cBox.addActionListener(this);
	revalidate();
	repaint();
    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JComboBox){ //AlgorithmSelectionPanel){
	    if(actionLeadToChange()){
		Weighter weighter = getSelectedWeighter();
		weighterFrame.setWeighter(weighter);
	    }
	}


    }


}


