package infomat.view.menu;

import infomat.view.*;

import javax.swing.*;

import java.awt.*;

import java.util.*;

/**
 * Displayer of the toolbar.
 */

public class ToolbarView extends JToolBar {

    private ToolbarController controller;
    private InfomatModel model;
    private HashMap<String, JToggleButton> buttons;
    private String pressed;

    private InfoPanel infoPanel;
    private String noText = " ";
	
    /*
      private JLabel infoLabel;
      private String noText = "<html> <br> </html>";
    */
	
    //private static String iconPath="E:/Java/Infomat/icons/";

    public Component add(Component comp, String name) {
	super.add(comp);
	try {
	    buttons.put(name, (JToggleButton) comp);
	} catch (ClassCastException e) {
	    // comp isn't a JToggleButton and it shouldn't be inserted in
	    // buttons
	}
	return comp;
    }

    public String getIconPath(){
	return (String) model.getPropertyValue("Icon Path");
    }
	
    /**
     * Create the buttons for the toolbar.
     * 
     * @param controller
     *            The catcher of all the events
     */
    public ToolbarView(ToolbarController controller, InfomatModel model) {
	super();
	setFloatable(false);
	setRollover(true);
	this.controller = controller;
	this.model=model;
	controller.setToolbarView(this);
	putAll();

    }

    public void putAll(){
	removeAll();

	buttons = new HashMap<String, JToggleButton>();

	/*
	  makeNavigationButton("save", ToolbarController.SAVE, "Save image",
	  "Save", false).setEnabled(false);
	*/
	/*
	  makeNavigationButton("overview", ToolbarController.OVERVIEW, 
	  "Toggle Overview", "ToggleOverview", true); //.setEnabled(true);
	  press(ToolbarController.OVERVIEW);
	*/
	//buttons.get(ToolbarController.OVERVIEW).doClick();
	makeNavigationButton("grid_vert", ToolbarController.GRIDCOLUMN,
			     "Toggle grid for columns", "Column Grid", true);
	makeNavigationButton("grid_horiz", ToolbarController.GRIDROW,
			     "Toggle grid for rows", "Row Grid", true);

	addSeparator();

	makeNavigationButton("pixel_select", ToolbarController.PIXELSELECTION,
			     "Pixel selection", "Pixel selection", true);
	makeNavigationButton("hand", ToolbarController.HAND,
			     "Drag", "Drag", true);
	makeNavigationButton("zoomselection", ToolbarController.ZOOMSELECTION,
			     "Zoom selection", "Zoom selection", true);
	/*
	makeNavigationButton("zoomin", ToolbarController.ZOOMIN,
			     "Zoom in/out with left/right-click",
			     "Zoom in/out with left/right-click", true);
	*/
	makeNavigationButton("delcol", ToolbarController.REMOVE_COLUMNS,
			     "Remove columns from grouping", "Remove columns", true);
	makeNavigationButton("delrow", ToolbarController.REMOVE_ROWS,
			     "Remove rows from grouping", "Remove rows", true);
	addSeparator();

	
	infoPanel = new InfoPanel();
	infoPanel.setText(noText, noText);
	add(infoPanel);

	revalidate();
	repaint();
	

	/*
	  infoLabel = new JLabel();
	  infoLabel.setText(noText);
	  add(infoLabel);
	*/
    }

    /*
      public void pressOverview(){
      buttons.get(ToolbarController.OVERVIEW).doClick();
      }
    */
	
    protected String press(String button) {
	System.out.println("Press " + button + " (pressed = " + pressed + ")");
	if (button == ToolbarController.PIXELSELECTION
	    || button == ToolbarController.HAND
	    || button == ToolbarController.ZOOMSELECTION
	    || button == ToolbarController.ZOOMIN
	    || button == ToolbarController.REMOVE_COLUMNS
	    || button == ToolbarController.REMOVE_ROWS) {
	    if (pressed != null && pressed != button) { // unclick the current
		// pressed button
		buttons.get(pressed).doClick();
	    }
	    pressed = button.equals(pressed) ? null : button;
	}
	return pressed;
    }

    protected void menuPress(String button) {
	buttons.get(button).doClick();
    }



    private AbstractButton makeNavigationButton(String imageName,
						String actionCommand, String toolTipText, String altText,
						Boolean toggle) {
	// Look for the image.
	String imgLocation = getIconPath() + imageName + ".gif";
	System.out.println(imgLocation);
		
	/*
	 * URL imageURL = null; try { imageURL = new URL(imgLocation);
	 * System.out.println("URL: "+imageURL); }catch(MalformedURLException
	 * e){}
	 */
	// Create and initialize the button.
	// JButton button = new JButton();
	AbstractButton button = toggle ? new JToggleButton() : new JButton();
	button.setActionCommand(actionCommand);
	button.setToolTipText(toolTipText);
	button.addActionListener(controller);

	// if (imageURL != null) { //image found
	button.setIcon(new ImageIcon(imgLocation, altText));
	/*
	 * } else { //no image found button.setText(altText);
	 * System.err.println("Resource not found: " + imgLocation); }
	 */

	add(button, actionCommand);

	return button;
    }

    /**
     * Set which word and document is pointed at
     * 
     * @param columnString
     *            The column that the mouse points at
     * @param rowString
     *            The row that the mouse points at
     */
    public void setInfo(String columnString, String rowString) {
	if (columnString == null && rowString == null)
	    infoPanel.setText(noText, noText);
	//infoLabel.setText(noText);
	else{
	    /*
	      if(columnString!=null)
	      if(columnString.length()>40)
	      columnString = columnString.substring(0,40)+" ...";
	      if(rowString!=null)
	      if(rowString.length()>40)
	      rowString = rowString.substring(0,40)+" ...";
	    */
	    infoPanel.setText(rowString, columnString);
			
	    //infoLabel.setText("<html>Row:"+ rowString + 
	    //		"<br>Column: " + columnString + "</html>");
	    //infoLabel.setText("<html><pre>Row: "+ rowString + 
	    //		"<br>Column: " + columnString + "</pre></html>");

	}
    }
	
    private class InfoPanel extends JPanel{
	private JLabel rowLabel;
	private JLabel colLabel;
		
	private String noText =" ";
		
	public InfoPanel(){
	    super();
	    setLayout(new GridLayout(2,1));
	    rowLabel = new JLabel(noText);
	    colLabel = new JLabel(noText);
	    setText(noText, noText);
	    add(rowLabel);
	    add(colLabel);
	}
		
	public void setText(String rowText, String colText){
	    rowLabel.setText("Row:        "+rowText);
	    colLabel.setText("Column:  "+colText);			
	    repaint();
	}
		
	public void paintComponent(Graphics g){
	    super.paintComponent(g);
	}
		
    }
	
	
}
