package infomat.view.picture;

import infomat.vectorspace.*;
import infomat.view.*;
import infomat.view.grouping.*;
import infomat.view.grouping.settings.GroupingColoring;
import mro.util.*;
import mro.util.doublehashmap.DoubleHashMap;

import java.util.*;
import java.awt.Color;
import java.awt.Dimension;

import mro.util.Printer;
import mro.util.TracePrinter;

public class SparsePictureConstructor{

    // The picture
    private static Dimension dimension;
    private static InfomatView view;
    //private static DoubleHashMap cellMap;

    // The Matrix
    private static IMatrix matrix;
    private static IObjectGrouping colGrouping;
    private static IObjectGrouping rowGrouping;
    // "From" is first element
    // "To" is the last plus one element
    // This should be consequent through the code.
    private static int colFrom;
    private static int colTo;
    private static int rowFrom;
    private static int rowTo;

    //Trace
    private static boolean traceOn=false;
    private static TracePrinter tp = new TracePrinter(traceOn);

    // Coloring
    /**
     * columnColoringKind
     *  0 - per column
     *  1 - per partPicture row and column
     */
    private static int columnColoringKind=0;

    public static SparsePicture zoom(InfomatView v,
				     IMatrix m, 
				     Dimension d, 
				     int cFrom, int cTo,
				     int rFrom, int rTo){
	/****************************/	
	view=v;
	matrix=m;
	colFrom = cFrom;
	colTo=cTo;
	rowFrom=rFrom;
	rowTo=rTo;
	dimension = 
	    new Dimension(d.width-SparsePicture.getGridSize(),
			  d.height-SparsePicture.getGridSize());
	colGrouping = matrix.getCurrentColumnGrouping();
	rowGrouping = matrix.getCurrentRowGrouping();
	//cellMap=matrix.getCellMap();

	//		Vector cells = matrix.getAllCells();
	if(colGrouping==null || colGrouping.numberOfIObjects()==0 ||
	   rowGrouping==null || rowGrouping.numberOfIObjects()==0){
	    //		   cells==null || cells.size()==0){

	    tp.println("****** returnerar null *********");
	    return null;

	}
	/****************************/	

	tp.println("New picture! (SparsePicture)");



	SparsePicture sp = 
	    initSparsePicture(); //partDimension, colFrom, colTo, rowFrom, rowTo);
	fillSparsePicture(sp); //, colFrom, colTo, rowFrom, rowTo);
	sp.makeCoordToPixelMap();

	sp.makeAverageColor(); //0.0f);

	return sp;
    }



    private static SparsePicture initSparsePicture(){ //Dimension d, 
	//int colFrom, int colTo,
	//int rowFrom, int rowTo){

	// The matrix
	int colWidth = colTo-colFrom;
	int rowHeight = rowTo-rowFrom;
	Vector<IObjectGroup> colGroups = 
	    colGrouping.getGroupsForIntervall(colFrom, colTo);
	Vector<IObjectGroup> rowGroups = 
	    rowGrouping.getGroupsForIntervall(rowFrom, rowTo);

	// The picture
	int width = (int) dimension.getWidth();
	int height = (int) dimension.getHeight();
	Vector<Integer> colLimits =
	    colGrouping.getGroupLimitsForIntervall(colFrom, colTo);
	Vector<Integer> rowLimits =
	    rowGrouping.getGroupLimitsForIntervall(rowFrom, rowTo);

	// The whole picture
	SparsePicture sp = 
	    new SparsePicture(matrix, dimension, colFrom, colTo, rowFrom, rowTo);

	// The part pictures
	SparsePartPicture[][] partPictureArray = new 
	    SparsePartPicture[colGroups.size()][rowGroups.size()];

	int currentOrigoY=0;
	int offsetY=0;
	for(int rgi=0; rgi<rowGroups.size(); rgi++){
	    IObjectGroup rowGroup = rowGroups.elementAt(rgi);
	    int partRowFrom = rowLimits.elementAt(rgi);
	    int partRowTo = rowLimits.elementAt(rgi+1);
	    int partRowHeight = partRowTo-partRowFrom;
	    int partHeight = 
		getImagePixels(partRowFrom, partRowTo, rowHeight, height);

	    int currentOrigoX=0;
	    int offsetX=0;
	    for(int cgi=0; cgi<colGroups.size(); cgi++){
		IObjectGroup colGroup = colGroups.elementAt(cgi);
		int partColFrom = colLimits.elementAt(cgi);
		int partColTo = colLimits.elementAt(cgi+1);			
		int partColWidth = partColTo-partColFrom;
		int partWidth = 
		    getImagePixels(partColFrom, partColTo, colWidth, width);

		// When the image is larger than the SparsePicture
		boolean largeImageRow = false;
		boolean largeImageCol = false;
		if(partWidth>partColWidth)
		    largeImageCol = true;
		if(partHeight>partRowHeight)
		    largeImageRow = true;
		    
		/**************************************/
		// The part picture
		Dimension partDimension = 
		    new Dimension(partWidth-SparsePicture.getGridSize(),
				  partHeight-SparsePicture.getGridSize());
		SparsePartPicture partSP =
		    new SparsePartPicture(partDimension, 
					  partColFrom, partColTo,
					  partRowFrom, partRowTo,
					  colGroup, rowGroup);
		partSP.setOrigo(currentOrigoX, currentOrigoY);
		partSP.setOffsetXY(offsetX, offsetY);
		partSP.setGridXY(cgi, rgi);
		partSP.setLargeImageRow(largeImageRow);
		partSP.setLargeImageCol(largeImageCol);
		partSP.setSparsePicture(sp);

		partPictureArray[cgi][rgi]=partSP;

		tp.print("[col, f: "+partColFrom+", t: "+partColTo+"; row, f: "+partRowFrom+", t: "+partRowTo+"], ");
		/**************************************/

		currentOrigoX += partWidth;
		offsetX--;
	    }
	    tp.println();
	    currentOrigoY += partHeight;
	    offsetY--;
	}

	sp.addPartPictureArray(partPictureArray);

	return sp;
    }


    private static int getImagePixels(int from, int to, int tot, int picTot){
	return (int)
	    //Math.round(
	    ((to-from)/((double) tot) * (picTot));
    }


    private static void fillSparsePicture(SparsePicture sparsePicture){ //, 
	//int colFrom, int colTo, int rowFrom, int rowTo){

	// The array of PartPictures
	SparsePartPicture[][] partPictureArray = 
	    sparsePicture.getPartPictureArray();

	// Colorings
	GroupingColoring rowColoring = 
	    view.getRowGroupingManager().getCurrentColoring();		
	GroupingColoring colColoring = 
	    view.getColumnGroupingManager().getCurrentColoring();		

	/******** Column preparation **********/
	// The x-coordinates for pixels
	// for all column PartPictures in partPictureArray
	HashMap<Integer, int[]> imageCoordLimitsForColumns = 
	    new HashMap<Integer, int[]>();

	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    SparsePartPicture spp = partPictureArray[cgi][0];
	    int partWidth = spp.getImageWidth();
	    int partColFrom = spp.getColFrom();
	    int partColTo = spp.getColTo();
	    int partNumOfCols = partColTo-partColFrom;

	    // The x-coordinates for pixels
	    int[] xs = makeLargeImageCoordLimits(partWidth, partNumOfCols); //+1);
	    //tp.println(cgi+", xs: "+intArrayToString(xs));
	    imageCoordLimitsForColumns.put(cgi, xs);

	    // Column Coloring
	    if(colColoring!=null){
		//Vector <SparsePixel> pixels = new Vector<SparsePixel>();
		//Color startColor = Color.black;
		//IMatrixCell startCell = new IMatrixCell(null, null, startWeight);
		for(int i=0; i<xs.length-1; i++){
		    for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
			SparsePartPicture colorSPP = partPictureArray[cgi][rgi];

			SparsePixel columnColorSP = 
			    new SparsePixel(xs[i], 0, 
					    xs[i+1], (int) colorSPP.getImageHeight(), 
					    null, null);
			colorSPP.putColumnColorPixel(columnColorSP);
			tp.print(xs[i]+", ");
		    }
		}
		tp.println();
	    }
	}
	/******** End of column preparation ******/


	// PartPictureArray, Row by row from the array
	for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
	    SparsePartPicture startSpp = partPictureArray[0][rgi];
	    int partRowFrom = startSpp.getRowFrom();
	    int partRowTo = startSpp.getRowTo();
	    int partHeight = startSpp.getImageHeight();
	    int partNumOfRows = partRowTo-partRowFrom;
	    // The y-coordinates for pixels
	    int[] ys = makeLargeImageCoordLimits(partHeight, partNumOfRows);

	    // Matrix, Row by row
	    for(int r=partRowFrom; r<partRowTo; r++){
		IObject rowObject = rowGrouping.getIObjectAt(r);

		// Row color
		Color rowColor = null;
		if(rowColoring!=null){
		    rowColor=rowColoring.getColorForObject(rowObject);
		}

		// What columns for this row?
		HashSet colCells = 
		    matrix.getColumnIMatrixCellsForRowObject(rowObject);
		if(colCells!=null){
		    Integer pixY = ys[r-partRowFrom];
		    Integer yLast = ys[r-partRowFrom+1]; 

		    // All existing columns for the row
		    Iterator colIterator = colCells.iterator();
		    while(colIterator.hasNext()){
			IMatrixCell mc = (IMatrixCell) colIterator.next();
			IObject colObject = mc.getColumnIObject();
			//int cgi = colGrouping.inWhichIObjectGroupOrdered(colObject);
			int c = colGrouping.getIObjectOrderIndex(colObject);
			int cgi = sparsePicture.getColumnPartPictureNumberForColumnIndex(c);

			if(cgi==-1 || rgi==-1){
			    tp.println("r: "+r);
			    tp.println("c: "+c);
			    tp.println("rgi: "+rgi);
			    tp.println("cgi: "+cgi);
			}

			if(cgi != -1){

			    SparsePartPicture spp = partPictureArray[cgi][rgi]; 
			    int partColFrom = spp.getColFrom();
			    int[] xs = imageCoordLimitsForColumns.get(cgi);
			    int pixX = xs[c-partColFrom];
			    if(c-partColFrom+1>xs.length-1){
				tp.println(Printer.intArrayToString(xs));
				tp.println(c-partColFrom+1);
				tp.println(cgi);
			    }

			    int xLast = xs[c-partColFrom+1];

			    // The pixel
			    SparsePixel sp = spp.getPixel(pixX, pixY);
			    SparsePixel newSp = 
				new SparsePixel(pixX, pixY, xLast, yLast, 
						rowColor, mc);
			    if(sp!=null)
				sp.accumulate(newSp);
			    else{
				spp.putPixel(newSp);
				sp=newSp;
			    }

			    // Column color
			    if(colColoring!=null){
				SparsePartPicture colorSPP=null;
				if(columnColoringKind==0){
				    // For per column
				    colorSPP = partPictureArray[cgi][0];
				}else if(columnColoringKind==1){
				    // For per row and column
				    colorSPP = partPictureArray[cgi][rgi];
				}
				Color colColor=colColoring.getColorForObject(colObject);
				SparsePixel columnColorSP = 
				    colorSPP.getColumnColorPixel(pixX);
				if(columnColorSP!=null){
				    tp.println("y-pos: "+(r-partRowFrom)+", pixY: "+pixY+", sx-pos: "+(c-partColFrom)+", pixX: "+pixX+", columnColorSP: "+columnColorSP);
				    SparsePixel ccNewSp = 
					new SparsePixel(columnColorSP.getX(), columnColorSP.getY(), 
							columnColorSP.getXLast(), columnColorSP.getYLast(), 
							colColor, mc);
				    columnColorSP.accumulate(ccNewSp);

				    /*
				      IMatrixCell dummyCell = 
				      new IMatrixCell(null, null, 0.1f); 
				      float ccWeight = dummyCell.getWeight();
				      tp.println("y-pos: "+(r-partRowFrom)+", pixY: "+pixY+", sx-pos: "+(c-partColFrom)+", pixX: "+pixX+", columnColorSP: "+columnColorSP);
				      SparsePixel ccNewSp = 
				      new SparsePixel(columnColorSP.getX(), columnColorSP.getY(), 
				      columnColorSP.getXLast(), columnColorSP.getYLast(), 
				      colColor, ccWeight, dummyCell);
				      columnColorSP.accumulateColorPixel(ccNewSp);

				    */

				}
			    }
			} // if(cgi != -1)

		    }
		}
	    }
	}

	// Average colors
	if(colColoring!=null){
	    sparsePicture.makeAverageColumnColor();
	    sparsePicture.adjustColumnColorOpacity(0.1f, 0.5f);
	}

	if(columnColoringKind==0){
	    // Set column coloring for all partPicture rows
	    if(partPictureArray[0].length>1){
		for(int rgi=1; rgi<partPictureArray[0].length; rgi++){
		    for(int cgi=0; cgi<partPictureArray.length; cgi++){
			SparsePartPicture spp = partPictureArray[cgi][rgi];
			spp.setColumnColorMap(partPictureArray[cgi][0].getColumnColorMap());
		    }
		}
	    }
	}



    }

    private static int[] makeLargeImageCoordLimits(int size, int numOfSteps){
	double step = ((double)size)/numOfSteps;
	int[] limits = new int[numOfSteps+1];
	limits[0]=0;
	for(int i=1; i<numOfSteps; i++)
	    limits[i]=(int) (i * step);
	limits[numOfSteps]=size;
	return limits;
    }




    /*

    private static void makeColumnColoring(SparsePicture sp){ //, int colFrom, int colTo){
    GroupingColoring colColoring = 
    view.getColumnColoringManager().getColoring();
    if(colColoring==null)
    return;

    SparsePartPicture[][] partPictureArray = 
    sp.getPartPictureArray();
    Vector<IObject> ordered = colGrouping.getOrdered();
    for(int cpp=0; cpp<partPictureArray.length; cpp++){
    SparsePartPicture spp = partPictureArray[cpp][0];
    Dimension d= spp.getImageSize();
    int colFrom = spp.getColFrom();
    int colTo = spp.getColTo();
    int numOfCols = colTo-colFrom;
    int[] xs = 
    makeLargeImageCoordLimits((int)d.getWidth(), numOfCols);
    HashMap<Integer, Integer> coordToOrder = 
    new HashMap<Integer, Integer>();
    for(int i=0; i<xs.length; i++){
    coordToOrder.put(xs[i], i);
    }

    Vector <SparsePixel> pixels = new Vector<SparsePixel>();
    Color startColor = Color.black; //null; //GroupingColoring.getStandardBackground();
    float startWeight = 0f;
    for(int i=0; i<xs.length-1; i++){
    pixels.add(new SparsePixel(xs[i], 0, 
    xs[i+1], (int)d.getHeight(), 
    startColor, startWeight, null));
    }

    for(int col=colFrom; col<colTo; col++){
    IObject io = ordered.elementAt(col);

    Color color=colColoring.getColorForObject(io);
    SparsePixel sPix = 
    pixels.elementAt(coordToOrder.get(xs[col-colFrom]));

    IMatrixCell dummyCell = 
    new IMatrixCell(null, null, 0.1f); 
    float weight = dummyCell.getWeight();
    SparsePixel newSp = 
    new SparsePixel(sPix.getX(), sPix.getY(), 
    sPix.getXLast(), sPix.getYLast(), 
    color, weight, dummyCell);
    sPix.accumulate(newSp);
    }


    int maxNumOfCells=0;
    for(int i=0; i<pixels.size(); i++){
    int numOfCells = pixels.elementAt(i).getNumberOfIMatrixCells();
    //tp.println("numOfCells: "+numOfCells);
    if(numOfCells>maxNumOfCells)
    maxNumOfCells=numOfCells;
    }
    float maxWeight=0.0f;
    float minWeight=Float.MAX_VALUE;
    for(int i=0; i<pixels.size(); i++){
    SparsePixel sPix = pixels.elementAt(i);
    sPix.divideValuesBy(maxNumOfCells);
    float weight=sPix.getWeight();
    if(weight>maxWeight)
    maxWeight=weight;
    if(weight<minWeight)
    minWeight=weight;
    }   	

    //tp.println("Color");
    //tp.println("Before: "+pixels.elementAt(0));
    //tp.print("Weight: ");
    for(int i=0; i<pixels.size(); i++){
    SparsePixel sPix = pixels.elementAt(i);
    sPix.divideValuesBy(maxNumOfCells);
    float weight=sPix.getWeight();
    //tp.print(sPix+", "); //weight+", ");
    }
    //tp.println();

    float newMinWeight = 0.1f;
    float newMaxWeight = 0.2f;
    spp.adjustWeight(pixels, newMinWeight, newMaxWeight, minWeight, maxWeight);
    for(int rpp=0; rpp<partPictureArray[0].length; rpp++){
    spp = partPictureArray[cpp][rpp];
    spp.setColumnColorPixels(new Vector<SparsePixel>(pixels));

    spp.adjustColumnColorPixelsToHeight();
    }

    //tp.println("Before: "+pixels.elementAt(0));
    //tp.print("Weight: ");
    for(int i=0; i<pixels.size(); i++){
    SparsePixel sPix = pixels.elementAt(i);
    sPix.divideValuesBy(maxNumOfCells);
    float weight=sPix.getWeight();
    //tp.print(sPix+", "); //weight+", ");
    }
    //tp.println();

    }
    }


    private static SparsePartPicture zoomUnGrouped(
    Dimension d, boolean largeImage,  
    int rowFrom, int rowTo,
    int colFrom, int colTo,
    IObjectGroup rowGroup, 
    IObjectGroup columnGroup){

    SparsePartPicture spp = 
    new SparsePartPicture(d, colFrom, colTo, rowFrom, rowTo, 
    columnGroup, rowGroup); //, view, matrix);
    // When the image is larger than the SparsePicture
    spp.setLargeImage(largeImage);

    int width = (int) d.getWidth();
    int height = (int) d.getHeight();
    int numOfCols = colTo-colFrom;
    int numOfRows = rowTo-rowFrom;
    int[] xs = makeLargeImageCoordLimits(width, numOfCols);
    int[] ys = makeLargeImageCoordLimits(height, numOfRows);

    IObjectGrouping colGrouping = 
    matrix.getCurrentColumnGrouping();
    IObjectGrouping rowGrouping = 
    matrix.getCurrentRowGrouping();

    GroupingColoring rowColoring = 
    view.getRowColoringManager().getColoring();		

    // Row by row
    for(int r=rowFrom; r<rowTo; r++){
    IObject rowObject = rowGrouping.getIObjectAt(r);
    HashSet colCells = 
    matrix.getColumnIMatrixCellsForRowObject(rowObject);
    if(colCells!=null){
    Integer pixY = ys[r-rowFrom];
    Integer yLast = ys[r-rowFrom+1]; 


    // Column by column
    Iterator colIterator = colCells.iterator();
    while(colIterator.hasNext()){
    IMatrixCell mc = (IMatrixCell) colIterator.next(); 
    IObject colObject = mc.getColumnIObject();
    int c = colGrouping.getIObjectOrderIndex(colObject);
    if(c>=colFrom && c<colTo){
    Integer pixX = xs[c-colFrom];
    Integer xLast = xs[c-colFrom+1];

    // The pixel
    SparsePixel sp = spp.getPixel(pixX, pixY);
    float weight = mc.getWeight();
    Color color = null;
    if(rowColoring!=null){
    color=rowColoring.getColorForObject(rowObject);
    }
    SparsePixel newSp = 
    new SparsePixel(pixX, pixY, xLast, yLast, 
    color, weight, mc);
    if(sp!=null)
    sp.accumulate(newSp);
    else
    spp.putPixel(newSp);
    }

    }
    }
    }

    SparsePixel.setNumberOfIMatrixColumns(1);
    SparsePixel.setNumberOfIMatrixRows(1);

    return spp;
    }

    private static SparsePartPicture oldZoomUnGrouped(Dimension d, 
    int colFrom, int colTo,
    int rowFrom, int rowTo,
    IObjectGroup columnGroup,
    IObjectGroup rowGroup){

    boolean test = true;
    if(test){
    boolean largeImage = true;

    }



    int width = (int) d.getWidth();
    int height = (int) d.getHeight();
    int numOfCols = colTo-colFrom;
    int numOfRows = rowTo-rowFrom;

    int colWidth = colTo-colFrom;
    int rowHeight = rowTo-rowFrom;

    SparsePartPicture spp = 
    new SparsePartPicture(d, colFrom, colTo, rowFrom, rowTo, 
    columnGroup, rowGroup); //, view, matrix);

    // When the image is larger than the SparsePicture
    if(width>numOfCols || height>numOfRows){
    spp.setLargeImage(true);

    int[] xs = makeLargeImageCoordLimits(width, numOfCols);
    int[] ys = makeLargeImageCoordLimits(height, numOfRows);

    //	    IObject[] cc = new IObject[colTo];

    // Optimering som skulle kunna gras:
    // numera kan man plocka "rad" ur DoubleHashMap.
    Vector cells = cellMap.values();
    for(int i=0; i<cells.size(); i++){
    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);

    // Hr borde man KANSKE ta fram r i frhllande till gruppen
    // som just behandlas, inte till hela setet.
    IObject rowObject = mc.getRowIObject();
    //int r = rows.getIObjectOrderIndex(rowObject);
    int r = rowGrouping.getIObjectOrderIndex(rowObject);

    //		int r = rows.getOrderNumberOfIObject(mc.getRowObject());
    //		int r = rowGroup.getIObjectOrderIndex(mc.getRowObject());

    if(r>=rowFrom && r<rowTo){
    IObject columnObject = mc.getColumnIObject();
    //int c = cols.getIObjectOrderIndex(columnObject);
    int c = colGrouping.getIObjectOrderIndex(columnObject);

    if(c>=colFrom && c<colTo){

    //			cc[c]=mc.getColumnObject();

    Integer pixX = xs[c-colFrom];
    Integer pixY = ys[r-rowFrom];
    Integer xLast = xs[c-colFrom+1];
    Integer yLast = ys[r-rowFrom+1]; 

    SparsePixel sp = spp.getPixel(pixX, pixY);

    GroupingColoring rowColoring = 
    view.getRowColoringManager().getColoring();
    Color color = null;
    if(rowColoring!=null){
    color=rowColoring.getColorForObject(rowObject);
    }
    float weight = mc.getWeight();
    SparsePixel newSp = 
    new SparsePixel(pixX, pixY, xLast, yLast, 
    color, weight, mc);
    if(sp!=null)
    sp.accumulate(newSp);
    else
    spp.putPixel(newSp);
    }

    }


    }

    SparsePixel.setNumberOfIMatrixColumns(1);
    SparsePixel.setNumberOfIMatrixRows(1);


    }else{
    // Optimering som skulle kunna gras:
    // numera kan man plocka "rad" ur DoubleHashMap.
    Vector cells = cellMap.values();


    // Det vore vl mycket effektivare att berkna vilka
    // MatrixCells man behver titta p och bara titta p dem?
    // - Nja, men kanske plocka rtt rader.
    // Man vill helst kunna hmta alla pixlar som ligger 
    // i en delmatris! Men det skulle krva mer datastrukturer
    // i IMatrix som behver updateras varje gng man ndrar
    // ngon ordning. 
    // En frbttring vore submatriser som svarar mot klustren.
    for(int i=0; i<cells.size(); i++){
    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
    IObject rowObject = mc.getRowIObject();
    //int r = rows.getIObjectOrderIndex(rowObject);
    int r = rowGrouping.getIObjectOrderIndex(rowObject);
    if(r>=rowFrom && r<rowTo){
    IObject columnObject = mc.getColumnIObject();
    //int c = cols.getIObjectOrderIndex(columnObject);
    int c = colGrouping.getIObjectOrderIndex(columnObject);
    if(c>=colFrom && c<colTo){
    Integer pixX = (int) 
    Math.round(((double) c-colFrom)/colWidth * (width-1));
    Integer pixY = (int) 
    Math.round(((double) r-rowFrom)/rowHeight * (height-1));
    SparsePixel sp = spp.getPixel(pixX, pixY);

    GroupingColoring rowColoring = 
    view.getRowColoringManager().getColoring();
    Color color = null;
    if(rowColoring!=null)
    color=rowColoring.getColorForObject(rowObject);

    tp.print("-");

    float weight = mc.getWeight();
    SparsePixel newSp = 
    new SparsePixel(pixX, pixY, color, weight, mc);
    if(sp!=null)
    sp.accumulate(newSp);
    else
    spp.putPixel(newSp);
    }
    }
    }

    SparsePixel.setNumberOfIMatrixColumns(((double) numOfCols)/width);
    SparsePixel.setNumberOfIMatrixRows(((double) numOfRows)/height);
    }


    return spp;
    }
    */
}
