package infomat.view.tools;

import infomat.algorithms.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.view.picture.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;

import javax.swing.border.*;
import javax.swing.filechooser.FileFilter;

import java.util.*;
import java.io.*;

import javax.swing.*;



public class ExportTextFrame extends JFrame implements ActionListener{

    public Dimension ALGORITHMFRAME_PREFERRED_SIZE = 
	new Dimension(300, 500);

    private IMatrix matrix;
    private InfomatModel infomatModel;
    private MainWindow mainWindow;

    private JButton rowsOrColumns;
    private JButton useRelative;
    private JButton apply;
    private JButton cancel;

    private JFileChooser fileChooser;

    //    private Properties properties;
    private IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();

	
    public ExportTextFrame(IMatrix matrix, InfomatModel infomatModel, MainWindow mainWindow){
	super("Export to Text");
	this.matrix=matrix;
	this.infomatModel = infomatModel;
	this.mainWindow=mainWindow;

	if(matrix==null){
	    dispose();
	    return;
	}

	String fileChooserPath = (String) infomatModel.getPropertyValue("Result Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });

	Container container = getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));


	//		apply = new JButton("Apply");
	//		JButton dummyButton = new JButton("Cancel");
	//		int height = (int) dummy.getPreferredSize().getHeight();
	//		int width = (int) dummyButton.getPreferredSize().getWidth();
	//		Dimension buttonSize = new Dimension(width, height);

	//		apply.addActionListener(this);
	//		add(apply);


	JTextField dummy = new JTextField("Apply");
	JButton dummyButton = new JButton("Cancel");
	int height = (int) dummy.getPreferredSize().getHeight();
	int width = (int) dummyButton.getPreferredSize().getWidth();
	Dimension buttonSize = new Dimension(width, height);

	JPanel infoAndButtons = new JPanel();
	infoAndButtons.setLayout(new GridLayout(4,1));
	JLabel info = new JLabel("Alter properties below and press \"Apply\"");
	infoAndButtons.add(info);

	JPanel dimPanel = new JPanel();
	dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
	JLabel dimInfo = new JLabel("Choose rows or columns: ");
	dimPanel.add(dimInfo);
	rowsOrColumns = new JButton("Rows");
	rowsOrColumns.setPreferredSize(buttonSize);
	rowsOrColumns.addActionListener(this);
	dimPanel.add(rowsOrColumns);
	infoAndButtons.add(dimPanel);

	JPanel relPanel = new JPanel();
	relPanel.setLayout(new BoxLayout(relPanel, BoxLayout.X_AXIS));
	JLabel relInfo = new JLabel("Use relative grouping as description: ");
	relPanel.add(relInfo);
	useRelative = new JButton("False");
	useRelative.setPreferredSize(buttonSize);
	useRelative.addActionListener(this);
	relPanel.add(useRelative);
	infoAndButtons.add(relPanel);

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new JButton("Apply");
	apply.setPreferredSize(buttonSize);
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new JButton("Cancel");
	cancel.setPreferredSize(buttonSize);
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	infoAndButtons.add(buttonPanel);
	infoAndButtons.setBorder(new TitledBorder("Export to Text"));
	container.add(infoAndButtons);


	/*
	  properties = new Properties("Export Properties");
	  Property p = new Property("Representation IObjects",0,"Representation IObjects, i.e the IObjects with the highest rank for the Groups.");
	  PropertyValueDescription pvd = new PropertyValueDescription("0","Relative Grouping, IObjects from the relative group.");
	  p.addValueDescription(pvd);
	  pvd = new PropertyValueDescription("1", "Centroid, IObjects from the centroid for the group.");
	  p.addValueDescription(pvd);
	  try{
	  properties.addProperty(p);
	  }catch (PropertyNameConflictException pnce){
	  //			never happens
	  }
	  String path = (String) model.getPropertyValue("Result Path");
	  PropertiesPanel pp = new PropertiesPanel(properties, path);
	*/
	
	String path = (String) infomatModel.getPropertyValue("Result Path");
	PropertiesPanel pp = new PropertiesPanel(gtio, path);
	container.add(pp);


	setVisible(true);
	setPreferredSize(ALGORITHMFRAME_PREFERRED_SIZE);
	pack();
    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == apply){
		System.out.println("Knappen intryckt!!");
		//				ProgressFrame pf = new ProgressFrame("Clustering - "+algorithm.getAlgorithmName());

		boolean row=true;
		if(rowsOrColumns.getText().equals("Columns"))
		    row=false;
		IObjectSet set = matrix.getColumnIObjectSet();
		IObjectSet relSet = matrix.getRowIObjectSet();
		if(row){
		    set = matrix.getRowIObjectSet();
		    relSet = matrix.getColumnIObjectSet();
		}

		fileChooser.setSelectedFile(new File(""));
		if (fileChooser.showOpenDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {
		    SparseISimilarity similarity = 
			new SparseDotSimilarity(matrix, row);
		    HashMap<IObjectGroup, IVector<IObject>> 
			descriptionVecMap = null;

		    boolean relative=true;
		    if(useRelative.getText().equals("False"))
			relative=false;
		    if(relative){
			IObjectGrouping grouping = 
			    set.getCurrentGrouping();
			IObjectGrouping relativeGrouping = 
			    relSet.getCurrentGrouping();
			if(grouping.numberOfIObjectGroups() != 
			   relativeGrouping.numberOfIObjectGroups()){
			    JOptionPane
				.showMessageDialog(
						   this,
						   "Wrong number of groups in relative grouping!",
						   "Export to Text",
						   JOptionPane.INFORMATION_MESSAGE);
			    return;
			}
				
			
		    
			descriptionVecMap = 
			    new HashMap<IObjectGroup, IVector<IObject>>();
			/*
			SparseISimilarity relSimilarity = 
			    new SparseDotSimilarity(matrix, !row);
			*/
			Vector<IObjectGroup> iogs = 
			    grouping.getGroups();
			Vector<IObjectGroup> relIOGs = 
			    relativeGrouping.getGroups();
			for(int i=0; i<relIOGs.size(); i++){
			    IObjectGroup group = iogs.elementAt(i);
			    IObjectGroup relGroup = relIOGs.elementAt(i);
			    IVector<IObject> descrVec = relGroup.getIVector();
			    descriptionVecMap.put(group, descrVec);
			}
		    }

		    gtio.save(matrix, set.getCurrentGrouping(), descriptionVecMap, similarity, fileChooser.getSelectedFile());

		    System.out.println("Sparat!!!");
		}

	    }else if(source == cancel){
		dispose();
	    }else if(source == rowsOrColumns){
		if(rowsOrColumns.getText().equals("Columns"))
		    rowsOrColumns.setText("Rows");
		else if(rowsOrColumns.getText().equals("Rows"))
		    rowsOrColumns.setText("Columns");
	    }else if(source == useRelative){
		if(useRelative.getText().equals("False"))
		    useRelative.setText("True");
		else if(useRelative.getText().equals("True"))
		    useRelative.setText("False");
	    }
	}
    }
}
