package infomat.view.vector;

import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.view.*;
import infomat.view.util.*;

import mro.util.*;
import mro.util.doublehashmap.DoubleHashMap;

import java.io.*;
import java.util.*;
import java.awt.Dimension;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;

import org.xml.sax.SAXException;


/**
 * A JPanel that presents the content of a IVector.
 * It gives the user several means of processing the content.
 * There are a number of methods with names beginning with "set"
 * that defines what of these methods are presented.
 *
 * @version          080805
 * @author           Magnus Rosell
 */
public class IVectorPanel<T extends IVectorIndex>
    extends JPanel implements ActionListener{

    // Hr planerar jag att fixa till s att man kan lsa in text-fil
    // till vilken IVector som helst.

    private InfomatModel model;

    private IVector<T> originalVec = new IVector<T>();
    private IVector<T> vec = new IVector<T>();
    private Vector<IVectorIndexPanel<T>> panelVec;

    private boolean selectable;
    private SmallButton selAllB;
    private SmallButton deselAllB;
    private SmallButton invSelB;
    private SmallButton rmB;

    private boolean fileable;
    private SmallButton loadB;
    private SmallButton saveB;
    private boolean loadable=true;
    private boolean saveable=true;
    private String fileChooserPath="";
    private JFileChooser fileChooser;
    private int fileType=0; // 0 - xml
                            // 1 - txt

    private boolean textFileable;
    private SmallButton textLoadB;
    private SmallButton textSaveB;
    private boolean textLoadable=true;
    private boolean textSaveable=true;
    private String textFileChooserPath="";
    private JFileChooser textFileChooser;

    private boolean orderable;
    private SmallButton originalOB;
    private SmallButton literalOB;
    private SmallButton inverseOB;
    private SmallButton normOB;

    //    private SmallButton orderApply;
    private OrderSelectionPanel orderSelectionPanel;


    private boolean showValues = true;

    private Stoplist stoplist=null;
    private IObjectGroup restrictionGroup;

    private ISimilarity similarity;
    private SmallButton simOB;

    public IVectorPanel(InfomatModel model){
	this.model=model;
    }

    public IVectorPanel(InfomatModel model, boolean selectable, boolean fileable){
	this.model=model;
	this.selectable=selectable;
	this.fileable=fileable;

	setFileChooser(); //fileChooserPath);
	initateGraphics();
    }

    public IVectorPanel(InfomatModel model, boolean selectable, IObjectGroup restrictionGroup){
	this.model=model;
	this.selectable=selectable;
	this.fileable=true;
	this.restrictionGroup=restrictionGroup;

	setFileChooser(); //fileChooserPath);
	initateGraphics();
    }

    private void setFileChooser(){//String fileChooserPath){
	fileChooser = new JFileChooser(fileChooserPath);

	if(fileType==0){
	    fileChooser.setFileFilter(new FileFilter() {
		    public String getDescription() {
			return "XML files";
		    }

		    public boolean accept(File f) {
			if (f.isDirectory())
			    return true;
			int findresult = f.getName().indexOf(".xml");
			int len = f.getName().length();
			return findresult != -1 && findresult == len - 4;
		    }

		});
	}else if(fileType==1){
	    fileChooser.setFileFilter(new FileFilter() {
		    public String getDescription() {
			return "TXT files";
		    }

		    public boolean accept(File f) {
			if (f.isDirectory())
			    return true;
			int findresult = f.getName().indexOf(".txt");
			int len = f.getName().length();
			return findresult != -1 && findresult == len - 4;
		    }

		});
	}

	textFileChooser = new JFileChooser(textFileChooserPath);
	textFileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "TXT files";
		}
		
		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".txt");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}
	    });
    }

    public void setXMLFiles(){
	stoplist=null;
	fileType=0;
	setFileChooser(); //fileChooserPath);
    }

    public void setTXTStopFiles(Stoplist stoplist){
	this.stoplist = stoplist;
	fileType=1;
	setFileChooser(); //fileChooserPath);
    }


    public void setSelectable(boolean selectable){
	this.selectable=selectable;
	putAll();
    }

    public void setFileable(boolean fileable){
	this.fileable=fileable;
	putAll();
    }

    public void setLoadSave(boolean loadable, boolean saveable){
	this.loadable=loadable;
	this.saveable=saveable;
	putAll();
    }

    public void setFilePath(String filePath){
	this.fileChooserPath = filePath;
	setFileChooser(); //filePath);
    }

    public void setTextFileable(boolean textFileable){
	this.textFileable=textFileable;
	putAll();
    }

    public void setTextLoadSave(boolean textLoadable, boolean textSaveable){
	this.textLoadable=textLoadable;
	this.textSaveable=textSaveable;
	putAll();
    }

    public void setTextFilePath(String textFilePath){
	this.textFileChooserPath = textFilePath;
	setFileChooser(); //filePath);
    }

    public void setOrderable(boolean orderable){
	this.orderable=orderable;
	putAll();
    }
    
    public boolean getOrderable(){
	return this.orderable;
    }    

    public void setShowValues(boolean showValues){
	this.showValues=showValues;
	putAll();
    }
  
    public void setRestrictionGroup(IObjectGroup restrictionGroup){
	this.restrictionGroup=restrictionGroup;
    }

    /**
     * Only works when the IVectorIndex:es are IObjects.
     * (When T is IObject.) Only implemented for SparseISimilarity yet.
     */
    public void setSimilarity(ISimilarity similarity){
	this.similarity = similarity;
	//	orderSelectionPanel = new OrderSelectionPanel(this);
	orderSelectionPanel.setSimilaritySort(true);
	putAll();
    }

    protected ISimilarity getSimilarity(){
	return similarity;
    }
    


    public void initateGraphics(){
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	loadB = new SmallButton("Load");
	loadB.addActionListener(this);
	saveB = new SmallButton("Save");
	saveB.addActionListener(this);

	textLoadB = new SmallButton("Load");
	textLoadB.addActionListener(this);
	textSaveB = new SmallButton("Save");
	textSaveB.addActionListener(this);

	selAllB = new SmallButton("Sel");
	selAllB.addActionListener(this);
	deselAllB = new SmallButton("Desel");
	deselAllB.addActionListener(this);
	invSelB = new SmallButton("Inv");
	invSelB.addActionListener(this);
	rmB = new SmallButton("Rm");
	rmB.addActionListener(this);

	originalOB = new SmallButton("Orig"); 
	originalOB.addActionListener(this);
	literalOB = new SmallButton("Lit"); 
	literalOB.addActionListener(this);
	inverseOB = new SmallButton("Inv"); 
	inverseOB.addActionListener(this);
	normOB = new SmallButton("Norm"); 
	normOB.addActionListener(this);
	simOB = new SmallButton("Sim"); 
	simOB.addActionListener(this);

	orderSelectionPanel = new OrderSelectionPanel(this);

	setIVector(vec);
    }

    public void setIVector(IVector<T> vec){
	setIVector(vec, new IVector<T>(vec));

	/*
	  this.vec=vec;
	  this.originalVec=vec;

	  panelVec = new Vector<IVectorIndexPanel<T>>();
	  Vector<T> ordered = vec.getOrdered();
	  for(int i=0; i<ordered.size(); i++){
	  T ivi = ordered.get(i);
	  IVectorIndexPanel<T> iip = 
	  new IVectorIndexPanel<T>(i+1, ivi, vec.getValue(ivi), selectable);
	  iip.setShowValue(showValues);
	  panelVec.add(iip);
	  }
	
	  putAll();
	*/
    }

    public void changeIVector(IVector<T> vec){
	setIVector(vec, originalVec);
    }
    
    private void setIVector(IVector<T> vec, 
			    IVector<T> originalVec){
	this.vec=vec;
	this.originalVec=originalVec;

	panelVec = new Vector<IVectorIndexPanel<T>>();
	Vector<T> ordered = vec.getOrderedVector();
	for(int i=0; i<ordered.size(); i++){
	    T ivi = ordered.get(i);
	    IVectorIndexPanel<T> iip = 
		new IVectorIndexPanel<T>(i+1, ivi, vec.getValue(ivi), selectable, model);
	    iip.setShowValue(showValues);
	    panelVec.add(iip);
	}
	
	putAll();
    }

    
    public IVector<T> getIVector(){
	return vec;
    }

    private IVector<T> getOriginalIVector(){
	return originalVec;
    }

    private void putAll(){
	removeAll();
	
	if(vec==null || panelVec==null)
	    return;

	if(fileable && (saveable || loadable)){
	    JPanel filePanel = new JPanel();
	    filePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel fileLabel = new JLabel("File: ");
	    if(fileType==1)
		fileLabel = new JLabel("Text File: ");

	    filePanel.add(fileLabel);
	    if(loadable)
		filePanel.add(loadB);
	    if(saveable)
		filePanel.add(saveB);
	    filePanel.setMinimumSize(filePanel.getPreferredSize());
	    filePanel.setMaximumSize(filePanel.getPreferredSize());
	    add(filePanel);
	    filePanel.setAlignmentX(LEFT_ALIGNMENT);
	}

	if(textFileable && (textSaveable || textLoadable)){
	    JPanel filePanel = new JPanel();
	    filePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel fileLabel = new JLabel("Text File: ");

	    filePanel.add(fileLabel);
	    if(textLoadable)
		filePanel.add(textLoadB);
	    if(textSaveable)
		filePanel.add(textSaveB);
	    filePanel.setMinimumSize(filePanel.getPreferredSize());
	    filePanel.setMaximumSize(filePanel.getPreferredSize());
	    add(filePanel);
	    filePanel.setAlignmentX(LEFT_ALIGNMENT);
	}

	if(selectable){
	    JPanel selectPanel = new JPanel();
	    selectPanel.setLayout(new BoxLayout(selectPanel, BoxLayout.Y_AXIS));

	    /*
	      JPanel allPanel = new JPanel();
	      allPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	      JLabel allLabel = new JLabel("All: ");
	      allPanel.add(allLabel);
	      allPanel.add(selAllB);
	      allPanel.add(deselAllB);
	      selectPanel.add(allPanel);

	      JPanel selectedPanel = new JPanel();
	      selectedPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	      JLabel selectedLabel = new JLabel("Selected: ");
	      selectedPanel.add(selectedLabel);
	      selectedPanel.add(rmB);
	      selectedPanel.add(invSelB);
	      selectPanel.add(selectedPanel);
	    */

	    JPanel selectedPanel = new JPanel();
	    selectedPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel selectedLabel = new JLabel("Sel: ");
	    selectedPanel.add(selectedLabel);
	    selectedPanel.add(selAllB);
	    selectedPanel.add(deselAllB);
	    selectedPanel.add(rmB);
	    selectedPanel.add(invSelB);
	    selectPanel.add(selectedPanel);

	    selectPanel.setMinimumSize(selectPanel.getPreferredSize());
	    selectPanel.setMaximumSize(selectPanel.getPreferredSize());

	    add(selectPanel);
	    selectPanel.setAlignmentX(LEFT_ALIGNMENT);
	}

	if(orderable){
	    /*
	      JPanel orderPanel = new JPanel();
	      orderPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	      JLabel orderLabel = new JLabel("Order: ");
	      orderPanel.add(orderLabel);
	      orderPanel.add(originalOB);
	      if(similarity!=null)
	      orderPanel.add(simOB);
	      orderPanel.add(literalOB);
	      orderPanel.add(inverseOB);
	      //	    orderPanel.add(normOB);
	      add(orderPanel);
	      orderPanel.setAlignmentX(LEFT_ALIGNMENT);
	    */

	    orderSelectionPanel.setMinimumSize(orderSelectionPanel.getPreferredSize());
	    orderSelectionPanel.setMaximumSize(orderSelectionPanel.getPreferredSize());
	    add(orderSelectionPanel);
	    orderSelectionPanel.setAlignmentX(LEFT_ALIGNMENT);

	}

	/*
	  if(fileable || selectable)
	  add(Box.createVerticalGlue());
	*/

	JScrollPane sp = new JScrollPane();
	JPanel indexPanel = new JPanel();
	indexPanel.setLayout(new BoxLayout(indexPanel, BoxLayout.Y_AXIS));
	for(int i=0; i<panelVec.size(); i++){
	    IVectorIndexPanel<T> iip = panelVec.get(i);
	    indexPanel.add(iip);
	}
	/*
	  indexPanel.add(Box.createVerticalGlue());
	  indexPanel.add(Box.createRigidArea(new Dimension(1,1)));
	*/
	sp.setViewportView(indexPanel);
	add(sp);
	sp.setAlignmentX(LEFT_ALIGNMENT);

	add(Box.createVerticalGlue());
	add(Box.createRigidArea(new Dimension(1,1)));


	revalidate();
	repaint();
    }

    public Vector<T> getSelectedIVectorIndexes(){
	Vector<T> vec = new Vector<T>();
	for(int i=0; i<panelVec.size(); i++){
	    IVectorIndexPanel<T> ivip = panelVec.get(i);
	    if(ivip.isSelected()){
		vec.add(ivip.getIVectorIndex());
	    }
	}
	return vec;
    }

    private void removeObjsFromVec(Vector<T> rm, IVector<T> vec){
	for(int i=0; i<rm.size(); i++){
	    T t = rm.get(i);
	    vec.remove(t);
	}
    }

    
    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == rmB){
		Vector<T> selected = getSelectedIVectorIndexes();
		if(selected.size()>0){
		    removeObjsFromVec(selected, vec);
		    removeObjsFromVec(selected, originalVec);
		    setIVector(vec, originalVec);
		}
		/*
		boolean removed = false;
		for(int i=0; i<panelVec.size(); i++){
		    IVectorIndexPanel<T> ivip = panelVec.get(i);
		    if(ivip.isSelected()){
			T ivi = ivip.getIVectorIndex();
			vec.remove(ivi);
			originalVec.remove(ivi);
			removed=true;
		    }
		}
		if(removed){
		    setIVector(vec, originalVec);
		}
		*/
	    }else if(source == selAllB){
		for(int i=0; i<panelVec.size(); i++){
		    IVectorIndexPanel<T> ivip = panelVec.get(i);
		    ivip.setSelected(true);
		}
	    }else if(source == deselAllB){
		for(int i=0; i<panelVec.size(); i++){
		    IVectorIndexPanel<T> ivip = panelVec.get(i);
		    ivip.setSelected(false);
		}
	    }else if(source == invSelB){
		for(int i=0; i<panelVec.size(); i++){
		    IVectorIndexPanel<T> ivip = panelVec.get(i);
		    ivip.setSelected(!ivip.isSelected());
		}
	    }else if(source == loadB){
		load();
	    }else if(source == saveB){
		save();
	    }else if(source == originalOB){
		setIVector(originalVec);
	    }else if(source == literalOB){
		vec.sortLiteral();
		setIVector(vec, originalVec);
	    }else if(source == inverseOB){
		vec.sortInvert();
		setIVector(vec, originalVec);
	    }else if(source == normOB){


	    }else if(source == simOB){
		// We know that the IVectorIndex:s are IObject:s

		IObjectGroup group = new IObjectGroup("tmp");
		Vector<T> vii = vec.getUnorderedVector();
		Vector<IObject> vio = new Vector<IObject>();
		for(int i=0; i<vii.size(); i++){
		    IObject io = (IObject) vii.get(i);
		    group.add(io);
		    vio.add(io);
		}
		
		IVector<T> newIVec = new IVector<T>();
		if(getSimilarity() instanceof SparseISimilarity){
		    SparseISimilarity sim = 
			(SparseISimilarity) getSimilarity();

		    IVector<IVectorIndex> groupRepr = 
			sim.getGroupRepresentation(group);

		    for(int i=0; i<vio.size(); i++){
			IObject io = vio.get(i);
			IVector<IVectorIndex> repr = 
			    sim.getRepresentation(io);
			newIVec.add((T) io, sim.sim(repr, groupRepr));
		    }
		}

		if(newIVec.size() > 0)
		    setIVector(newIVec, originalVec);

	    }

	}
    }



    private void load(){
		
	fileChooser.setDialogTitle("Load");
	fileChooser.setSelectedFile(new File(""));
	if (fileChooser.showDialog(getParent(), "Load") == JFileChooser.APPROVE_OPTION) {
	    File file = fileChooser.getSelectedFile();

	    if(fileType==0){
		IVectorIO<T> ivio = new IVectorIO<T>();
		IVector<T> newVec = 
		    ivio.load(restrictionGroup, file);
		vec.removeAll();
		vec.add(newVec);

		setIVector(vec);    
	    }else if(fileType==1){
		stoplist.readFile(file);

		//		setIVector(stoplist.getStopStringsVector());

		// Bad solution!!!
		IVector<IVectorString> ivec = stoplist.getStopStringsVector();
		IVector<T> nvec = new IVector<T>();
		Vector<IVectorString> vec = ivec.getUnorderedVector();
		for(int i=0; i<vec.size(); i++){
		    IVectorString ss = vec.get(i);
		    nvec.add((T) ss, ivec.getValue(ss));
		}
		setIVector(nvec);

	    }


	}
    }


    public void save(){
	fileChooser.setDialogTitle("Save");
	fileChooser.setSelectedFile(new File(""));
	if (fileChooser.showDialog(getParent(), "Save") == JFileChooser.APPROVE_OPTION) {
	    IVectorIO<T> ivio = new IVectorIO<T>();
	    try{
		File file = fileChooser.getSelectedFile();
		
		if(fileType==0){
		    if(!file.getCanonicalPath().endsWith(".xml"))
			file = new File(file.getCanonicalPath()+".xml");
		    ivio.save(vec, file);
		}else if(fileType==1){
		    if(!file.getCanonicalPath().endsWith(".txt"))
			file = new File(file.getCanonicalPath()+".txt");
		    if(stoplist==null){
			showMessage("No stoplist", "Tried to load a stoplist without a stoplist-object");
			return;
		    }
		    stoplist.writeFile(file);
		}


	    }catch(IOException ioe){
		showMessage("IOException", "An IOException occured when trying to save.");
	    }
	}
    }


    /*
      private class SmallButton extends JButton{

      public SmallButton(String string){
      super(string);
      JTextField dummy = new JTextField(string);
      JButton dummyButton = new JButton(string);
      int height = (int) dummy.getPreferredSize().getHeight();
      int width = (int) dummyButton.getPreferredSize().getWidth();
      Dimension buttonSize = new Dimension(width, height);
      setPreferredSize(buttonSize);
      }

      }
    */


    private void showMessage(String title, String message){
	JOptionPane.showMessageDialog(
				      this, message, title, 
				      JOptionPane.INFORMATION_MESSAGE);

    }


    private class OrderSelectionPanel 
	extends JPanel implements ActionListener{

	private IVectorPanel ivp;
	private SmallButton orderApply;
	private JComboBox cBox;

	private Vector<String> sortNames;
	private String selected;

	boolean similaritySortable=false;

	private static final String SORT_SIMILARITY_TO_SELECTED = "Sim. to Sel.";
	private static final String SORT_SIMILARITY = "Similarity";
	private static final String SORT_LITERAL = "Literal";
	private static final String SORT_RANDOM = "Random";
	private static final String SORT_INVERT = "Invert";
	private static final String SORT_ORIGINAL = "Original";

	public OrderSelectionPanel(IVectorPanel ivp){
	    this.ivp=ivp;
	    orderApply = new SmallButton("Apply"); 
	    orderApply.addActionListener(this);

	    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

	    selected = SORT_LITERAL;

	    putAll();
	}

	public void setSimilaritySort(boolean similaritySortable){
	    this.similaritySortable = similaritySortable;
	    putAll();
	}

	public void putAll(){
	    removeAll();

	    sortNames = new Vector<String>();

	    if(similaritySortable){
		sortNames.add(SORT_SIMILARITY_TO_SELECTED);
		sortNames.add(SORT_SIMILARITY);
	    }
	    sortNames.add(SORT_LITERAL);
	    sortNames.add(SORT_RANDOM);
	    sortNames.add(SORT_INVERT);
	    sortNames.add(SORT_ORIGINAL);


	    JPanel orderPanel = new JPanel();
	    orderPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel orderLabel = new JLabel("Order: ");
	    orderPanel.add(orderLabel);
	    
	    cBox = new JComboBox(sortNames);
	    cBox.setSelectedItem(selected);
	    orderPanel.add(cBox);

	    orderPanel.add(orderApply);

	    add(orderPanel);
	    //	    orderPanel.setAlignmentX(LEFT_ALIGNMENT);
	    revalidate();
	    repaint();
	}

	public boolean actionLeadToChange(){
	    String newSelected = (String) cBox.getSelectedItem();
	    //	System.out.println("newSelected: "+newSelected);
	    if(selected != newSelected){
		selected = newSelected;
		return true;
	    }
	    return false;
	}




	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JButton){
		JButton source = (JButton) e.getSource();
		if(source == orderApply){
		    boolean removed = false;

		    String sortName = (String) cBox.getSelectedItem();

		    System.out.println(sortName);

		    if(sortName.equals(SORT_ORIGINAL)){
			ivp.setIVector(ivp.getOriginalIVector());
		    }else if(sortName.equals(SORT_LITERAL)){
			ivp.getIVector().sortLiteral();
			ivp.setIVector(getIVector(), getOriginalIVector());
		    }else if(sortName.equals(SORT_INVERT)){
			ivp.getIVector().sortInvert();
			ivp.setIVector(ivp.getIVector(), ivp.getOriginalIVector());
		    }else if(sortName.equals(SORT_RANDOM)){
			ivp.getIVector().sortRandom();
			ivp.setIVector(ivp.getIVector(), ivp.getOriginalIVector());

		    }else if(sortName.equals(SORT_SIMILARITY)){
			// We know that the IVectorIndex:s are IObject:s

			IObjectGroup group = new IObjectGroup("tmp");
			Vector<T> vii = ivp.getIVector().getUnorderedVector();
			Vector<IObject> vio = new Vector<IObject>();
			for(int i=0; i<vii.size(); i++){
			    IObject io = (IObject) vii.get(i);
			    group.add(io);
			    vio.add(io);
			}
		
			IVector<T> newIVec = new IVector<T>();
			if(ivp.getSimilarity() instanceof SparseISimilarity){
			    SparseISimilarity sim = (SparseISimilarity) ivp.getSimilarity();

			    IVector<IVectorIndex> groupRepr = 
				sim.getGroupRepresentation(group);

			    for(int i=0; i<vio.size(); i++){
				IObject io = vio.get(i);
				IVector<IVectorIndex> repr = 
				    sim.getRepresentation(io);
				newIVec.add((T) io, sim.sim(repr, groupRepr));
			    }
			}

			if(newIVec.size() > 0)
			    ivp.setIVector(newIVec, ivp.getOriginalIVector());

		    }else if(sortName.equals(SORT_SIMILARITY_TO_SELECTED)){
			// We know that the IVectorIndex:s are IObject:s

			Vector<T> vii = ivp.getSelectedIVectorIndexes();
			IObjectGroup group = new IObjectGroup("tmp");
			for(int i=0; i<vii.size(); i++){
			    IObject io = (IObject) vii.get(i);
			    group.add(io);
			}

			vii = ivp.getIVector().getUnorderedVector();
			Vector<IObject> vio = new Vector<IObject>();
			for(int i=0; i<vii.size(); i++){
			    IObject io = (IObject) vii.get(i);
			    vio.add(io);
			}
		
			IVector<T> newIVec = new IVector<T>();
			if(ivp.getSimilarity() instanceof SparseISimilarity){
			    SparseISimilarity sim = 
				(SparseISimilarity) ivp.getSimilarity();

			    IVector<IVectorIndex> groupRepr = 
				sim.getGroupRepresentation(group);

			    for(int i=0; i<vio.size(); i++){
				IObject io = vio.get(i);
				IVector<IVectorIndex> repr = 
				    sim.getRepresentation(io);
				newIVec.add((T) io, sim.sim(repr, groupRepr));
			    }
			}

			if(newIVec.size() > 0)
			    ivp.setIVector(newIVec, ivp.getOriginalIVector());
		    }
		}
	    }
	    else if(e.getSource() instanceof JComboBox){ 
		actionLeadToChange();
	    }
	}



    }

}


