package mro.util;

import java.io.*;
import java.util.Vector;

/**
 * A helper class. Some functions used when printing 
 * Latex-documents.
 *
 * @version          080325
 * @author           Magnus Rosell
 */
public class LatexHelper{

    public static void printLatexHead(PrintWriter pw) throws IOException{
        pw.println("\\documentclass[11pt, a4paper]{article}\n\\usepackage[T1]{fontenc}\n\\usepackage[english]{babel}\n\\usepackage[latin1]{inputenc}\n\\usepackage[dvips]{graphicx}\n\\usepackage{amssymb}\n\\usepackage{verbatim}\n");
        pw.println("\\begin{document}\n\n");
    }


    public static void 
	printTableHead(PrintWriter pw,
		       Vector<String> columnTitles,
		       boolean rowTitles) throws IOException{
        
	pw.println("\\begin{table*}[thb]");
	
	pw.print("\\begin{tabular}{l");
	if(rowTitles)
	    pw.print("|l");
	for(int i=0; i<columnTitles.size()-1; i++)
	    pw.print("|l");
	pw.println("}");

	if(rowTitles)
	    pw.print("&"+columnTitles.elementAt(0));
	else
	    pw.print(columnTitles.elementAt(0));
	for(int i=1; i<columnTitles.size(); i++)
	    pw.print("&"+columnTitles.elementAt(i));
	pw.println("\\\\");
	printHLine(pw);
    }
    
    public static void 
	printTableHead(PrintWriter pw,
		       int numberOfColumns) throws IOException{
	printTableHead(pw, numberOfColumns, 0);
    }

    public static void 
	printTableHead(PrintWriter pw,
		       int numberOfColumns,
		       int textSize) throws IOException{
        
	pw.println("\\begin{table*}[thb]");
	
	printBeginTextSize(pw, textSize);

	pw.print("\\begin{tabular}{r");
	for(int i=0; i<numberOfColumns-1; i++)
	    pw.print("|r");
	pw.println("|}");
	//	printHLine(pw);
    }
    
    public static void printHLine(PrintWriter pw){
	pw.println("\\hline");
    }

    public static void printTablePartHorizontalLine(PrintWriter pw, 
						    String multiString){
	pw.println("\\cline{"+multiString+"}");
    }

    public static void 
	printTableRow(PrintWriter pw, 
		      Vector<String> values) throws IOException{

	pw.print(values.elementAt(0)+" ");
	for(int i=1; i<values.size(); i++)
	    pw.print("&"+values.elementAt(i)+" ");
	pw.println("\\\\");
    }


    public static void 
	printTableEnd(PrintWriter pw,
		      String caption, 
		      String label) throws IOException{
	printTableEnd(pw, caption, label, 0);
    }


    public static void 
	printTableEnd(PrintWriter pw,
		      String caption, 
		      String label,
		      int textSize) throws IOException{

	pw.println("\\end{tabular}");
	printEndTextSize(pw, textSize);
	pw.println("\\caption{"+caption+"}");
	pw.println("\\label{"+label+"}");
	pw.println("\\end{table*}");
    }

    public static void printTableEnd(PrintWriter pw) throws IOException{
	printTableEnd(pw, 0);
    }


    public static void printTableEnd(PrintWriter pw, 
				     int textSize) throws IOException{
	pw.println("\\end{tabular}");
	printEndTextSize(pw, textSize);
	pw.println("\\end{table*}");
    }

    public static void printLatexEnd(PrintWriter pw) throws IOException{
        pw.println("\n\n\\end{document}");
    }

    public static void printBeginTextSize(PrintWriter pw, 
					  int textSize) throws IOException{
	if(textSize>0){
	    if(textSize==1)
		pw.println("\\begin{small}");	    
	    else if(textSize==2)
		pw.println("\\begin{footnote}");
	    else if(textSize==3)
		pw.println("\\begin{sciptsize}");
	}
    }

    public static void printEndTextSize(PrintWriter pw, 
					int textSize) throws IOException{
	if(textSize>0){
	    if(textSize==1)
		pw.println("\\end{small}");
	    else if(textSize==2)
		pw.println("\\end{footnote}");
	    else if(textSize==3)
		pw.println("\\end{sciptsize}");
	}
    }


}
